# -*- coding: utf-8 -*-
"""Setup tests for this package."""
from iosanita.contenttypes.testing import INTEGRATION_TESTING
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from z3c.relationfield import RelationValue
from zope.component import getUtility
from zope.event import notify
from zope.intid.interfaces import IIntIds
from zope.lifecycleevent import ObjectModifiedEvent

import unittest


class TestStrutturaCorrelata(unittest.TestCase):
    """"""

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        intids = getUtility(IIntIds)

        self.struttura1 = api.content.create(
            container=self.portal, type="Struttura", title="struttura 1"
        )
        self.struttura2 = api.content.create(
            container=self.portal, type="Struttura", title="struttura 2"
        )
        self.struttura_test = api.content.create(
            container=self.portal,
            type="Struttura",
            title="Test struttura",
        )
        self.servizio_test = api.content.create(
            container=self.portal,
            type="Servizio",
            title="Test servizio",
        )
        self.struttura_test.struttura_correlata = [
            RelationValue(intids.getId(self.struttura1))
        ]
        self.servizio_test.struttura_correlata = [
            RelationValue(intids.getId(self.struttura1)),
            RelationValue(intids.getId(self.struttura2)),
        ]
        notify(ObjectModifiedEvent(self.struttura_test))
        notify(ObjectModifiedEvent(self.servizio_test))

    def test_struttura_correlata_reference_in_struttura_is_in_catalog(self):
        """ """
        res = api.content.find(struttura_correlata_uid=self.struttura1.UID())

        self.assertEqual(len(res), 2)
        uids = [x.UID for x in res]
        self.assertIn(self.struttura_test.UID(), uids)

    def test_struttura_correlata_reference_in_servizio_is_in_catalog(self):
        """ """
        res = api.content.find(struttura_correlata_uid=self.struttura1.UID())

        self.assertEqual(len(res), 2)
        uids = [x.UID for x in res]
        self.assertIn(self.servizio_test.UID(), uids)

        res = api.content.find(struttura_correlata_uid=self.struttura2.UID())

        self.assertEqual(len(res), 1)
        self.assertEqual(self.servizio_test.UID(), res[0].UID)



# -*- coding: utf-8 -*-
from collective.volto.blocksfield.field import BlocksField
from iosanita.contenttypes import _
from plone.app.dexterity import textindexer
from plone.app.z3cform.widget import RelatedItemsFieldWidget
from plone.autoform import directives as form
from plone.autoform.interfaces import IFormFieldProvider
from plone.dexterity.interfaces import IDexterityContent
from plone.supermodel import model
from z3c.relationfield.schema import RelationChoice
from z3c.relationfield.schema import RelationList
from zope.component import adapter
from zope.interface import implementer
from zope.interface import provider


@provider(IFormFieldProvider)
class IEvento(model.Schema):
    """Marker inteerface for content type Evento"""

    descrizione_estesa = BlocksField(
        title=_("descrizione_estesa_label", default="Cos'è"),
        required=True,
        description=_(
            "evento_descrizione_estesa_help",
            default="Descrizione dettagliata e completa.",
        ),
    )

    costo = BlocksField(
        title=_("costo_label", default="Costo"),
        required=True,
        description=_(
            "costo_help",
            default="Eventuale costo dell'evento (se ci sono uno o più biglietti), "
            "con link all'acquisto se disponibile.",
        ),
    )

    organizzato_da_interno = RelationList(
        title=_(
            "organizzato_da_interno_label",
            default="Organizzato da: Unità organizzativa",
        ),
        default=[],
        value_type=RelationChoice(vocabulary="plone.app.vocabularies.Catalog"),
        required=False,
        description=_(
            "organizzato_da_interno_help",
            default="Se l'evento è organizzato da un'Unità organizzativa dell'ASL, includere collegamento alla relativa pagina.",
        ),
    )
    organizzato_da_esterno = BlocksField(
        title=_("organizzato_da_esterno_label", default="Organizato da: altro"),
        required=False,
        description=_(
            "organizzato_da_esterno_help",
            default="Indicazione di chi orgnanizza l'evento, se diverso da un'Unità organizzativa.",
        ),
    )

    #  campi aggiunti con il pnrr
    patrocinato_da = BlocksField(
        title=_("patrocinato_da_label", default="Patrocinato da"),
        required=False,
        description=_(
            "patrocinato_da_help",
            default="Indicare l'ente che supporta l'evento, se presente.",
        ),
    )

    persona_correlata = RelationList(
        title=_("persona_correlata_event_label", default="Parteciperanno (Persone)"),
        required=False,
        default=[],
        value_type=RelationChoice(vocabulary="plone.app.vocabularies.Catalog"),
        description=_(
            "persona_correlata_event_help",
            default="Seleziona una serie di Persone dell'ASL che interverranno all'evento.",
        ),
    )
    parteciperanno = BlocksField(
        title=_("parteciperanno_label", default="Parteciperanno"),
        required=False,
        description=_(
            "parteciperanno_help",
            default="Lista delle persone che parteciperanno, se non fanno parte dell'ASL.",
        ),
    )

    eventi_correlati = RelationList(
        title="Eventi correlati",
        default=[],
        required=False,
        description=_(
            "eventi_correlati_help",
            default="Seleziona altri eventi simili o collegati.",
        ),
        value_type=RelationChoice(vocabulary="plone.app.vocabularies.Catalog"),
    )

    webinar = BlocksField(
        title=_("webinar_label", default="Webinar"),
        required=False,
        description=_(
            "webinar_help",
            default="Se l'evento è un webinar, specificare che è online e includere il link alla piattaforma di streaming.",
        ),
    )
    struttura_correlata = RelationList(
        title=_("struttura_correlata_label", default="Struttura di riferimento"),
        description=_(
            "struttura_correlata_help",
            default="Indicare una o più strutture che fanno a capo a questo contenuto.",
        ),
        value_type=RelationChoice(vocabulary="plone.app.vocabularies.Catalog"),
        default=[],
        required=False,
    )

    # custom order
    form.order_after(organizzato_da_interno="pdc_correlato")
    form.order_after(organizzato_da_esterno="organizzato_da_interno")
    form.order_after(patrocinato_da="organizzato_da_esterno")

    # custom widgets

    form.widget(
        "eventi_correlati",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "selectableTypes": ["Event"],
        },
    )
    form.widget(
        "organizzato_da_interno",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "selectableTypes": ["UnitaOrganizzativa"],
        },
    )
    form.widget(
        "persona_correlata",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "selectableTypes": ["Persona"],
        },
    )
    form.widget(
        "struttura_correlata",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={"selectableTypes": ["Struttura"]},
    )

    # custom fieldsets and order
    model.fieldset(
        "cosa_e",
        label=_("cosa_e_fieldset", default="Cos'è"),
        fields=[
            "descrizione_estesa",
        ],
    )

    model.fieldset("costi", label=_("costi_label", default="Costi"), fields=["costo"])
    model.fieldset(
        "contatti",
        label=_("contatti_label", default="Contatti"),
        description=_(
            "contatti_fieldset_help",
            default='Almeno uno dei due campi "Organizzato da" deve essere compilato.',
        ),
        fields=[
            "organizzato_da_interno",
            "organizzato_da_esterno",
            "patrocinato_da",
        ],
    )
    model.fieldset(
        "partecipanti",
        label=_("partecipanti_label", default="Chi partecipa"),
        fields=["persona_correlata", "parteciperanno"],
    )
    model.fieldset(
        "contenuti_collegati",
        label=_("contenuti_collegati_label", default="Contenuti collegati"),
        fields=["eventi_correlati"],
    )

    model.fieldset(
        "dove",
        label=_("dove_label", default="Dove"),
        description=_(
            "dove_event_help",
            default="Se la sede di questo contenuto non coincide con la Struttura di riferimento o non è un webinar, compilare gli altri campi.",
        ),
        fields=["webinar", "struttura_correlata"],
    )
    textindexer.searchable("descrizione_estesa")


@implementer(IEvento)
@adapter(IDexterityContent)
class Evento(object):
    """ """

    def __init__(self, context):
        self.context = context



# -*- coding: utf-8 -*-
from iosanita.contenttypes import _
from iosanita.contenttypes.interfaces import IIosanitaContenttypes
from plone.supermodel import model
from zope import schema


class ICartellaModulistica(model.Schema, IIosanitaContenttypes):
    """Cartella Modulistica"""

    anteprima_file = schema.Bool(
        title=_("anteprima_file_title", default="Mostra i PDF in anteprima"),
        description=_(
            "anteprima_file_description",
            default="Permette di aprire l'anteprima di tutti i PDF di questa cartella"
            " in una tab separata, altrimenti i PDF vengono scaricati",
        ),
        required=False,
        default=False,
    )

    ricerca_in_testata = schema.Bool(
        title=_("ricerca_in_testata_label", default="Ricerca in testata"),
        default=False,
        required=False,
        description=_(
            "ricerca_in_testata_help",
            default="Seleziona se mostrare o meno il campo di ricerca in testata.",
        ),
    )



# -*- coding: utf-8 -*-
"""Setup tests for this package."""
from iosanita.contenttypes.testing import INTEGRATION_TESTING
from plone import api
from plone.app.dexterity.behaviors.metadata import IDublinCore
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.formwidget.geolocation.geolocation import Geolocation
from plone.indexer.interfaces import IIndexableObject
from z3c.relationfield import RelationValue
from zope.component import getUtility
from zope.component import queryMultiAdapter
from zope.event import notify
from zope.intid.interfaces import IIntIds
from zope.lifecycleevent import Attributes
from zope.lifecycleevent import ObjectModifiedEvent

import unittest


class TestCustomIndexes(unittest.TestCase):
    """"""

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        self.catalog = api.portal.get_tool(name="portal_catalog")
        self.intids = getUtility(IIntIds)

    def test_parent_metadata_has_parent_data(self):
        parent = api.content.create(
            container=self.portal,
            type="Document",
            title="Parent",
        )

        child = api.content.create(
            container=parent,
            type="Document",
            title="Child",
        )

        brain = api.content.find(UID=child.UID())[0]

        self.assertEqual(
            sorted(["title", "UID", "@id", "@type"]), sorted(list(brain.parent.keys()))
        )
        self.assertEqual(brain.parent["title"], "Parent")
        self.assertEqual(brain.parent["UID"], parent.UID())

    def test_when_parent_changes_children_will_be_updated(self):
        parent = api.content.create(
            container=self.portal,
            type="Document",
            title="Parent",
        )

        child = api.content.create(
            container=parent,
            type="Document",
            title="Child",
        )

        brain = api.content.find(UID=child.UID())[0]
        self.assertEqual(brain.parent["title"], "Parent")

        # update title
        parent.title = "New Parent Title"
        descriptions = [Attributes(IDublinCore, *["IDublinCore.title"])]
        notify(ObjectModifiedEvent(parent, *descriptions))

        brain = api.content.find(UID=child.UID())[0]
        self.assertEqual(brain.parent["title"], "New Parent Title")

    def test_has_geolocation_false_if_geolocation_is_not_set(self):
        item = api.content.create(
            container=self.portal,
            type="Struttura",
            title="Struttura",
        )
        adapter = queryMultiAdapter((item, self.catalog), IIndexableObject)
        self.assertFalse(adapter.has_geolocation)

    def test_has_geolocation_false_if_geolocation_is_set_as_0(self):
        item = api.content.create(
            container=self.portal,
            type="Struttura",
            title="Struttura",
        )
        adapter = queryMultiAdapter((item, self.catalog), IIndexableObject)
        self.assertFalse(adapter.has_geolocation)

        item.geolocation = Geolocation(latitude=0, longitude=0)

        item.reindexObject()

        adapter = queryMultiAdapter((item, self.catalog), IIndexableObject)
        self.assertFalse(adapter.has_geolocation)

    def test_has_geolocation_true_if_geolocation_is_set(self):
        item = api.content.create(
            container=self.portal,
            type="Struttura",
            title="Struttura",
        )
        adapter = queryMultiAdapter((item, self.catalog), IIndexableObject)
        self.assertFalse(adapter.has_geolocation)

        item.geolocation = Geolocation(latitude=10, longitude=10)

        item.reindexObject()

        self.assertTrue(adapter.has_geolocation)

    def test_has_geolocation_for_persona_false_if_has_no_dove_fields_compiled(
        self,
    ):
        item = api.content.create(
            container=self.portal,
            type="Persona",
            title="Persona",
        )
        adapter = queryMultiAdapter((item, self.catalog), IIndexableObject)
        self.assertFalse(adapter.has_geolocation)

    def test_has_geolocation_for_persona_false_if_geolocation_is_set_as_0(self):
        item = api.content.create(
            container=self.portal,
            type="Persona",
            title="Persona",
        )
        adapter = queryMultiAdapter((item, self.catalog), IIndexableObject)
        self.assertFalse(adapter.has_geolocation)

        item.geolocation = Geolocation(latitude=0, longitude=0)

        item.reindexObject()

        adapter = queryMultiAdapter((item, self.catalog), IIndexableObject)
        self.assertFalse(adapter.has_geolocation)

    def test_has_geolocation_for_persona_true_if_geolocation_is_set(self):
        item = api.content.create(
            container=self.portal,
            type="Persona",
            title="Persona",
        )
        adapter = queryMultiAdapter((item, self.catalog), IIndexableObject)
        self.assertFalse(adapter.has_geolocation)

        item.geolocation = Geolocation(latitude=10, longitude=10)

        item.reindexObject()

        adapter = queryMultiAdapter((item, self.catalog), IIndexableObject)
        self.assertTrue(adapter.has_geolocation)

    def test_has_geolocation_for_persona_true_if_struttura_in_cui_opera_is_set(self):
        struttura = api.content.create(
            container=self.portal, type="Struttura", title="Struttura"
        )
        item = api.content.create(
            container=self.portal,
            type="Persona",
            title="Persona",
        )
        adapter = queryMultiAdapter((item, self.catalog), IIndexableObject)
        self.assertFalse(adapter.has_geolocation)

        item.struttura_in_cui_opera = [RelationValue(self.intids.getId(struttura))]

        notify(ObjectModifiedEvent(item))
        item.reindexObject()

        adapter = queryMultiAdapter((item, self.catalog), IIndexableObject)
        self.assertTrue(adapter.has_geolocation)

    def test_has_geolocation_for_persona_true_if_struttura_ricevimento_is_set(self):
        struttura = api.content.create(
            container=self.portal, type="Struttura", title="Struttura"
        )
        item = api.content.create(
            container=self.portal,
            type="Persona",
            title="Persona",
        )
        adapter = queryMultiAdapter((item, self.catalog), IIndexableObject)
        self.assertFalse(adapter.has_geolocation)

        item.struttura_ricevimento = [RelationValue(self.intids.getId(struttura))]

        notify(ObjectModifiedEvent(item))
        item.reindexObject()

        adapter = queryMultiAdapter((item, self.catalog), IIndexableObject)
        self.assertTrue(adapter.has_geolocation)

    def test_geolocation_metadata_is_empty_if_no_geolocation_set(self):
        struttura = api.content.create(
            container=self.portal,
            type="Struttura",
            title="Struttura",
        )

        brain = api.content.find(UID=struttura.UID())[0]
        self.assertEqual(brain.geolocation, {})

    def test_geolocation_metadata_has_geolocation_data(self):
        struttura = api.content.create(
            container=self.portal,
            type="Struttura",
            title="Struttura",
            geolocation=Geolocation(latitude=10, longitude=20),
        )

        brain = api.content.find(UID=struttura.UID())[0]
        self.assertEqual(brain.geolocation["latitude"], struttura.geolocation.latitude)
        self.assertEqual(
            brain.geolocation["longitude"], struttura.geolocation.longitude
        )



# -*- coding: utf-8 -*-
from Acquisition import aq_inner
from iosanita.contenttypes.interfaces import IoSanitaViewExtraData
from iosanita.contenttypes.interfaces.persona import IPersona
from iosanita.contenttypes.interfaces.servizio import IServizio
from iosanita.contenttypes.interfaces.struttura import IStruttura
from iosanita.contenttypes.interfaces.unita_organizzativa import IUnitaOrganizzativa
from plone.restapi.interfaces import ISerializeToJsonSummary
from redturtle.bandi.interfaces.bando import IBando
from zc.relation.interfaces import ICatalog
from zope.component import adapter
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.globalrequest import getRequest
from zope.interface import implementer
from zope.interface import Interface
from zope.intid.interfaces import IIntIds
from zope.security import checkPermission


LIMIT = 25


@implementer(IoSanitaViewExtraData)
@adapter(Interface, Interface)
class ViewExtraDataExtractor(object):
    reference_id = None

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def __call__(self):
        """
        By default does not return anything
        """
        return {}

    def get_back_references(self, reference_id):
        """
        Return a mapping of back references splitted by portal_type.
        News Items are sorted by date, other are sorted by title
        """

        catalog = getUtility(ICatalog)
        intids = getUtility(IIntIds)
        relations = []
        for ref_id in reference_id:
            relations.extend(
                catalog.findRelations(
                    {
                        "to_id": intids.getId(aq_inner(self.context)),
                        "from_attribute": ref_id,
                    }
                )
            )
        data = {}
        for rel in relations:
            obj = intids.queryObject(rel.from_id)
            if obj is None:
                continue
            portal_type = obj.portal_type
            if checkPermission("zope2.View", obj):
                if portal_type not in data:
                    data[portal_type] = []
                summary = getMultiAdapter(
                    (obj, getRequest()), ISerializeToJsonSummary
                )()
                if summary not in data[portal_type]:
                    data[portal_type].append(summary)
        for portal_type, values in data.items():
            if portal_type == "News Item":
                data[portal_type] = sorted(
                    values, key=lambda k: k["Date"], reverse=True
                )[:LIMIT]
            else:
                data[portal_type] = sorted(values, key=lambda k: k["title"])[:LIMIT]
        return data


@implementer(IoSanitaViewExtraData)
@adapter(IServizio, Interface)
class ViewExtraDataExtractorServizio(ViewExtraDataExtractor):
    def __call__(self):
        """
        Servizio can also be referenced by a custom field
        """

        data = self.get_back_references(reference_id=["servizio_correlato"])

        data.update(
            self.get_back_references(reference_id=["servizio_procedura_riferimento"])
        )
        return {"back-references": data}


@implementer(IoSanitaViewExtraData)
@adapter(IStruttura, Interface)
class ViewExtraDataExtractorStruttura(ViewExtraDataExtractor):
    def __call__(self):
        """ """
        return {
            "back-references": self.get_back_references(
                reference_id=["struttura_correlata"]
            )
        }


@implementer(IoSanitaViewExtraData)
@adapter(IUnitaOrganizzativa, Interface)
class ViewExtraDataExtractorUnitaOrganizzativa(ViewExtraDataExtractor):
    def __call__(self):
        """ """
        return {
            "back-references": self.get_back_references(
                reference_id=["uo_correlata", "struttura_correlata"]
            )
        }


@implementer(IoSanitaViewExtraData)
@adapter(IPersona, Interface)
class ViewExtraDataExtractorPersona(ViewExtraDataExtractor):
    def __call__(self):
        data = self.get_back_references(reference_id=["persona_correlata"])

        # append additional references
        data.update(
            {
                "responsabile": self.get_back_references(
                    reference_id=["responsabile_correlato"]
                ),
                "personale": self.get_back_references(
                    reference_id=["personale_correlato"]
                ),
            }
        )
        return {"back-references": data}


@implementer(IoSanitaViewExtraData)
@adapter(IBando, Interface)
class ViewExtraDataExtractorBando(ViewExtraDataExtractor):
    def __call__(self):
        bando_view = self.context.restrictedTraverse("bando_view")
        return {
            "approfondimenti": self.get_approfondimenti(),
            "stato_bando": bando_view.getBandoState(),
        }

    def get_approfondimenti(self):
        """ """
        folders = self.context.listFolderContents(
            contentFilter={"portal_type": "Bando Folder Deepening"}
        )

        result = []

        for folder in folders:
            if folder.exclude_from_nav:
                continue
            folder_data = getMultiAdapter(
                (folder, self.request), ISerializeToJsonSummary
            )()
            items = []
            for child in folder.listFolderContents():
                if child.exclude_from_nav:
                    continue
                child_data = getMultiAdapter(
                    (child, self.request), ISerializeToJsonSummary
                )()
                # if child.portal_type == "Link":
                #     url = getattr(child, "remoteUrl", "") or ""

                #     if url.startswith("${portal_url}/resolveuid/"):
                #         uid = url.replace("${portal_url}/", "")
                #         child_data["@id"] = uid_to_url(uid)
                items.append(child_data)
            if items:
                folder_data["items"] = items
                result.append(folder_data)
        return result



# -*- coding: utf-8 -*-
from iosanita.contenttypes.interfaces import IIosanitaContenttypesLayer
from iosanita.contenttypes.utils import create_default_blocks
from plone import api
from Products.CMFPlone.interfaces import ISelectableConstrainTypes

import logging


logger = logging.getLogger(__name__)


SUBFOLDERS_MAPPING = {
    "ComeFarePer": {
        "content": [
            {"id": "allegati", "title": "Allegati", "allowed_types": ("File",)},
        ],
    },
    "Bando": {
        "content": [
            {
                "id": "graduatoria",
                "title": "Graduatoria",
                "type": "Bando Folder Deepening",
            },
            {
                "id": "altri-allegati",
                "title": "Altri allegati",
                "type": "Bando Folder Deepening",
            },
            {
                "id": "adempimenti-consequenziali",
                "title": "Adempimenti consequenziali",
                "type": "Bando Folder Deepening",
            },
        ],
    },
    "Documento": {
        "content": [
            {
                "id": "immagini",
                "title": "Immagini",
                "type": "Document",
                "allowed_types": ("Image", "Link"),
            },
        ],
    },
    "Event": {
        "content": [
            {
                "id": "immagini",
                "title": "Immagini",
                "allowed_types": ("Image", "Link"),
                "publish": True,
            },
            {
                "id": "video",
                "title": "Video",
                "allowed_types": ("Link",),
                "publish": True,
            },
            {
                "id": "sponsor-evento",
                "title": "Sponsor Evento",
                "allowed_types": ("Link",),
                "publish": True,
            },
            {
                "id": "allegati",
                "title": "Allegati",
                "allowed_types": ("File",),
                "publish": True,
            },
        ],
    },
    "News Item": {
        "content": [
            {
                "id": "immagini",
                "title": "Immagini",
                "allowed_types": ("Image", "Link"),
            },
            {
                "id": "video",
                "title": "Video",
                "allowed_types": ("Link",),
            },
            {
                "id": "allegati",
                "title": "Allegati",
                "allowed_types": ("File",),
            },
        ],
    },
    "Persona": {
        "content": [
            {
                "id": "curriculum-vitae",
                "title": "Curriculum vitae",
                "allowed_types": ("File",),
            },
            {
                "id": "immagini",
                "title": "Immagini",
                "allowed_types": ("Image", "Link"),
            },
            {
                "id": "video",
                "title": "Video",
                "allowed_types": ("Link",),
            },
            {
                "id": "allegati",
                "title": "Allegati",
                "allowed_types": ("File",),
            },
        ],
        "allowed_types": [],
    },
    "Servizio": {
        "content": [
            {
                "id": "modulistica",
                "title": "Modulistica",
                "allowed_types": (
                    "File",
                    "Link",
                ),
            },
            {
                "id": "allegati",
                "title": "Allegati",
                "allowed_types": ("File",),
            },
        ],
    },
    "UnitaOrganizzativa": {
        "content": [
            {"id": "allegati", "title": "Allegati", "allowed_types": ("File",)},
        ],
    },
    "Step": {
        "content": [
            {"id": "allegati", "title": "Allegati", "allowed_types": ("File",)},
        ],
    },
    "Struttura": {
        "content": [
            {"id": "allegati", "title": "Allegati", "allowed_types": ("File",)},
            {
                "id": "immagini",
                "title": "Immagini",
                "allowed_types": ("Image", "Link"),
            },
            {
                "id": "video",
                "title": "Video",
                "allowed_types": ("Link",),
            },
        ],
    },
}


def onModify(context, event):
    for description in event.descriptions:
        if "IBasic.title" in getattr(
            description, "attributes", []
        ) or "IDublinCore.title" in getattr(description, "attributes", []):
            context_state = api.content.get_view(
                name="plone_context_state", context=context, request=context.REQUEST
            )
            if context_state.is_folderish():
                for child in context.listFolderContents():
                    child.reindexObject(idxs=["parent"])


def createSubfolders(context, event):
    """
    Create subfolders structure based on a portal_type mapping
    """
    if not IIosanitaContenttypesLayer.providedBy(context.REQUEST):
        return

    subfolders_mapping = SUBFOLDERS_MAPPING.get(context.portal_type, [])
    if not subfolders_mapping:
        return

    for mapping in subfolders_mapping.get("content", {}):
        if mapping["id"] not in context.keys():
            portal_type = mapping.get("type", "Document")
            child = api.content.create(
                container=context,
                type=portal_type,
                title=mapping["title"],
                id=mapping["id"],
            )

            if portal_type == "Document":
                create_default_blocks(context=child)

            if portal_type in ["Folder", "Document", "Bando Folder Deepening"]:
                child.exclude_from_search = True
                child.reindexObject(idxs=["exclude_from_search"])
                if portal_type != "Bando Folder Deepening":
                    child.exclude_from_nav = True
                    child.reindexObject(idxs=["exclude_from_nav"])

            # select constraints
            if mapping.get("allowed_types", ()):
                constraints_child = ISelectableConstrainTypes(child)
                constraints_child.setConstrainTypesMode(1)
                constraints_child.setLocallyAllowedTypes(mapping["allowed_types"])

            if mapping.get("publish", False):
                with api.env.adopt_roles(["Reviewer"]):
                    api.content.transition(obj=child, transition="publish")

    allowed_types = subfolders_mapping.get("allowed_types", None)
    if allowed_types is not None and not isinstance(allowed_types, list):
        raise ValueError("Subfolder map is not well formed")

    if isinstance(allowed_types, list):
        constraints_context = ISelectableConstrainTypes(context)
        constraints_context.setConstrainTypesMode(1)
        constraints_context.setLocallyAllowedTypes(allowed_types)



from iosanita.contenttypes.interfaces import IIosanitaContenttypesLayer
from iosanita.contenttypes.interfaces.settings import IIoSanitaContenttypesSettings
from iosanita.contenttypes.interfaces.settings import (
    IIoSanitaContenttypesSettingsControlpanel,
)
from plone.restapi.controlpanels import RegistryConfigletPanel
from zope.component import adapter
from zope.interface import implementer
from zope.interface import Interface


@adapter(Interface, IIosanitaContenttypesLayer)
@implementer(IIoSanitaContenttypesSettingsControlpanel)
class IoSanitaContenttypesSettingsControlpanel(RegistryConfigletPanel):
    schema = IIoSanitaContenttypesSettings
    configlet_id = "IoSanitaContenttypesSettings"
    configlet_category_id = "Products"
    schema_prefix = None



# -*- coding: utf-8 -*-
from collective.taxonomy.interfaces import ITaxonomy
from plone import api
from Products.CMFPlone.interfaces import INonInstallable
from redturtle.bandi.interfaces.settings import IBandoSettings
from zope.component import getUtilitiesFor
from zope.interface import implementer

import logging


logger = logging.getLogger(__name__)
DEFAULT_PROFILE = "profile-iosanita.contenttypes:default"


@implementer(INonInstallable)
class HiddenProfiles(object):
    def getNonInstallableProfiles(self):
        """Hide uninstall profile from site-creation and quickinstaller."""
        return [
            "iosanita.contenttypes:uninstall",
        ]

    def getNonInstallableProducts(self):
        """Hide the upgrades package from site-creation and quickinstaller."""
        return ["iosanita.contenttypes.upgrades"]


class colors(object):
    GREEN = "\033[92m"
    ENDC = "\033[0m"
    RED = "\033[91m"
    DARKCYAN = "\033[36m"
    YELLOW = "\033[93m"


def update_profile(context, profile, run_dependencies=True):
    context.runImportStepFromProfile(DEFAULT_PROFILE, profile, run_dependencies)


def update_types(context):
    update_profile(context, "typeinfo")


def update_registry(context):
    update_profile(context, "plone.app.registry", run_dependencies=False)


def update_catalog(context):
    update_profile(context, "catalog")


def post_install(context):
    """Post install script"""

    # add taxonomies
    context.runImportStepFromProfile(
        "iosanita.contenttypes:taxonomy", "collective.taxonomy"
    )

    # setup bandi
    setup_bandi()


def setup_bandi():
    """
    clean bandi data
    """
    # remove default ente
    api.portal.set_registry_record("default_ente", (), interface=IBandoSettings)
    api.portal.set_registry_record(
        "tipologie_bando",
        (("Bando|Bando", "Concorso|Concorso")),
        interface=IBandoSettings,
    )


def post_install_taxonomy(context):
    for utility_name, utility in list(getUtilitiesFor(ITaxonomy)):
        utility.updateBehavior(**{"field_prefix": ""})
        logger.info(
            f"{colors.DARKCYAN} Change taxonomy prefix for {utility_name} {colors.ENDC}"  # noqa
        )

    logger.info(
        f"{colors.DARKCYAN} iosanita.contentypes taxonomies imported {colors.ENDC}"  # noqa
    )
    update_catalog(context)


def uninstall(context):
    """Uninstall script"""
    # Do something at the end of the uninstallation of this package.



# -*- coding: utf-8 -*-
from iosanita.contenttypes import _
from iosanita.contenttypes.interfaces import IIosanitaContenttypesLayer
from plone.dexterity.interfaces import IDexterityContent
from plone.formwidget.geolocation.geolocation import Geolocation
from plone.formwidget.geolocation.interfaces import IGeolocationField
from plone.restapi.deserializer.dxfields import DefaultFieldDeserializer
from plone.restapi.interfaces import IFieldDeserializer
from zope.component import adapter
from zope.i18n import translate
from zope.interface import implementer


@implementer(IFieldDeserializer)
@adapter(IGeolocationField, IDexterityContent, IIosanitaContenttypesLayer)
class GeolocationFieldDeserializer(DefaultFieldDeserializer):
    def __call__(self, value):
        if "latitude" not in value or "longitude" not in value:
            raise ValueError(
                translate(
                    _(
                        "geolocation_field_validator_label",
                        default="Invalid geolocation data: ${value}. Provide latitude and longitude coordinates.",  # noqa
                        mapping={"value": value},
                    ),
                    context=self.request,
                )
            )
        return Geolocation(latitude=value["latitude"], longitude=value["longitude"])



# -*- coding: utf-8 -*-
from iosanita.contenttypes.interfaces.persona import IPersona
from plone.dexterity.interfaces import IDexterityContent
from plone.indexer.decorator import indexer


def get_geolocation_data(context):
    geolocation = getattr(context.aq_base, "geolocation", None)
    if not geolocation:
        return {}
    return {
        "latitude": geolocation.latitude,
        "longitude": geolocation.longitude,
    }


@indexer(IDexterityContent)
def parent(context):
    obj_parent = context.aq_parent
    return {
        "title": obj_parent.Title(),
        "UID": obj_parent.UID(),
        "@id": obj_parent.absolute_url(),
        "@type": obj_parent.portal_type,
    }


@indexer(IDexterityContent)
def exclude_from_search(context):
    return getattr(context.aq_base, "exclude_from_search", False)


@indexer(IDexterityContent)
def street(context):
    return getattr(context.aq_base, "street", None)


@indexer(IDexterityContent)
def zip_code(context):
    return getattr(context.aq_base, "zip_code", None)


@indexer(IDexterityContent)
def city(context):
    return getattr(context.aq_base, "city", None)


@indexer(IDexterityContent)
def provincia(context):
    return getattr(context.aq_base, "provincia", None)


@indexer(IDexterityContent)
def geolocation(context):
    return get_geolocation_data(context)


@indexer(IDexterityContent)
def has_geolocation(context):
    data = getattr(context.aq_base, "geolocation", None)
    if not data:
        return False
    if data.latitude == 0.0 and data.longitude == 0.0:
        return False
    return True


@indexer(IPersona)
def has_geolocation_persona(context):
    """
    Persona can have several locations
    """
    struttura_in_cui_opera = getattr(context.aq_base, "struttura_in_cui_opera", [])
    struttura_ricevimento = getattr(context.aq_base, "struttura_ricevimento", [])
    if struttura_in_cui_opera or struttura_ricevimento:
        return True
    data = getattr(context.aq_base, "geolocation", None)
    if not data:
        return False
    if data.latitude == 0.0 and data.longitude == 0.0:
        return False
    return True



# -*- coding: utf-8 -*-

from collective.volto.blocksfield.field import BlocksField
from iosanita.contenttypes import _
from iosanita.contenttypes.interfaces import IIosanitaContenttypes
from plone.app.z3cform.widget import RelatedItemsFieldWidget
from plone.autoform import directives as form
from plone.supermodel import model
from z3c.relationfield.schema import RelationChoice
from z3c.relationfield.schema import RelationList
from zope import schema


class IDocumento(model.Schema, IIosanitaContenttypes):
    """Marker interface for content type"""

    protocollo = schema.TextLine(
        title=_(
            "protocollo_documento_label",
            default="Numero di protocollo",
        ),
        description=_(
            "protocollo_documento_help",
            default="Il numero di protocollo del documento.",
        ),
        max_length=255,
        required=False,
    )

    data_protocollo = schema.Date(
        title=_("data_protocollo", default="Data del protocollo"),
        required=False,
    )

    descrizione_estesa = BlocksField(
        title=_("descrizione_estesa_label", default="Cos'è"),
        required=True,
        description=_(
            "descrizione_estesa_documento_help",
            default="Descrizione estesa e completa del Documento.",
        ),
    )

    servizio_procedura_riferimento = RelationList(
        title=_(
            "servizio_procedura_riferimento_label",
            default="Servizio di riferimento / Procedura di riferimento",
        ),
        description=_(
            "servizio_procedura_riferimento_help",
            default="Indicazione del servizio, la  prestazione o la procedura (Come fare per) a cui fa riferimento il documento.",
        ),
        default=[],
        value_type=RelationChoice(vocabulary="plone.app.vocabularies.Catalog"),
        required=False,
        missing_value=(),
    )

    uo_correlata = RelationList(
        title=_("uo_correlata_documento_label", default="Responsabile del documento"),
        description=_(
            "uo_correlata_documento_help",
            default="Indicazione dell'unità organizzativa responsabile del documento.",
        ),
        default=[],
        value_type=RelationChoice(vocabulary="plone.app.vocabularies.Catalog"),
        required=True,
        missing_value=(),
    )

    autori = RelationList(
        title=_(
            "autori_label",
            default="Autore/i",
        ),
        description=_(
            "autori_help",
            default="Seleziona una lista di autori che hanno pubblicato "
            "il documento.",
        ),
        value_type=RelationChoice(vocabulary="plone.app.vocabularies.Catalog"),
        required=False,
        default=[],
    )

    # widgets
    form.widget(
        "uo_correlata",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "maximumSelectionSize": 1,
            "selectableTypes": ["UnitaOrganizzativa"],
        },
    )
    form.widget(
        "autori",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "selectableTypes": ["Persona"],
        },
    )
    form.widget(
        "servizio_procedura_riferimento",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "selectableTypes": ["ComeFarePer", "Servizio"],
        },
    )

    model.fieldset(
        "cosa_e",
        label=_("cosa_e_fieldset", default="Cos'è"),
        fields=["descrizione_estesa"],
    )

    model.fieldset(
        "riferimenti",
        label=_("riferimenti_fieldset", default="Riferimenti"),
        fields=["servizio_procedura_riferimento", "uo_correlata", "autori"],
    )



# -*- coding: utf-8 -*-
from collective.taxonomy import PATH_SEPARATOR
from collective.taxonomy.interfaces import ITaxonomy
from plone.dexterity.interfaces import IDexterityContent
from plone.indexer.decorator import indexer
from zope.component import getUtility
from zope.globalrequest import getRequest


def get_taxonomy_vocab(field):
    request = getRequest()
    taxonomy = getUtility(ITaxonomy, name=f"collective.taxonomy.{field}")
    return taxonomy.makeVocabulary(request.get("LANGUAGE"))


def extract_taxonomies(context, field, only_leaf=False):
    taxonomy_voc = get_taxonomy_vocab(field)
    data = []
    value = getattr(context, field, []) or []
    if not isinstance(value, list):
        value = [value]
    for key in value:
        taxonomy_value = taxonomy_voc.inv_data.get(key, None)
        if not taxonomy_value:
            continue
        if taxonomy_value.startswith(PATH_SEPARATOR):
            taxonomy_value = taxonomy_value.replace(PATH_SEPARATOR, "", 1)

        if only_leaf:
            data.append(
                {"title": taxonomy_value.split(PATH_SEPARATOR)[-1], "token": key}
            )
        else:
            data.append(
                {"title": taxonomy_value.replace(PATH_SEPARATOR, "", 1), "token": key}
            )
    return data


@indexer(IDexterityContent)
def parliamo_di_metadata(context, **kw):
    """ """
    return extract_taxonomies(context=context, field="parliamo_di")


@indexer(IDexterityContent)
def a_chi_si_rivolge_tassonomia_metadata(context, **kw):
    """ """
    return extract_taxonomies(context=context, field="a_chi_si_rivolge_tassonomia")


@indexer(IDexterityContent)
def incarico_metadata(context, **kw):
    """ """
    return extract_taxonomies(context=context, field="incarico", only_leaf=True)


@indexer(IDexterityContent)
def tipologia_notizia_metadata(context, **kw):
    """ """
    return extract_taxonomies(
        context=context, field="tipologia_notizia", only_leaf=True
    )


@indexer(IDexterityContent)
def tipologia_servizio_metadata(context, **kw):
    """ """
    return extract_taxonomies(
        context=context, field="tipologia_servizio", only_leaf=True
    )



# -*- coding: utf-8 -*-
"""Setup tests for this package."""
from iosanita.contenttypes.testing import INTEGRATION_TESTING
from iosanita.contenttypes.testing import RESTAPI_TESTING
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.restapi.testing import RelativeSession
from Products.CMFPlone.interfaces import ISelectableConstrainTypes

import unittest


class TestUOSchema(unittest.TestCase):
    layer = RESTAPI_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)

    def tearDown(self):
        self.api_session.close()

    def test_behaviors_enabled_for_uo(self):
        portal_types = api.portal.get_tool(name="portal_types")
        self.assertEqual(
            portal_types["UnitaOrganizzativa"].behaviors,
            (
                "plone.namefromtitle",
                "plone.allowdiscussion",
                "plone.excludefromnavigation",
                "plone.shortname",
                "plone.ownership",
                "plone.publication",
                "plone.categorization",
                "iosanita.basic",
                "iosanita.contenttypes.behavior.sottotitolo",
                "plone.locking",
                "plone.leadimage",
                "volto.preview_image",
                "plone.relateditems",
                "plone.textindexer",
                "plone.translatable",
                "kitconcept.seo",
                "plone.versioning",
                "iosanita.contenttypes.behavior.dove_required",
                "iosanita.contenttypes.behavior.contatti",
                "iosanita.contenttypes.behavior.ulteriori_informazioni",
            ),
        )

    def test_uo_fieldsets(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/UnitaOrganizzativa").json()
        self.assertEqual(len(resp["fieldsets"]), 13)
        self.assertEqual(
            [x.get("id") for x in resp["fieldsets"]],
            [
                "default",
                "cosa_fa",
                "persone_uo",
                "dove",
                "orari",
                "contatti",
                "documenti",
                "ulteriori_informazioni",
                "settings",
                "ownership",
                "dates",
                "categorization",
                "seo",
            ],
        )

    def test_uo_required_fields(self):
        resp = self.api_session.get("@types/UnitaOrganizzativa").json()
        self.assertEqual(
            sorted(resp["required"]),
            sorted(
                [
                    "city",
                    "competenze",
                    "country",
                    "description",
                    "orari",
                    "pdc_correlato",
                    "provincia",
                    "street",
                    "title",
                    "zip_code",
                ]
            ),
        )

    def test_uo_fields_default_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/UnitaOrganizzativa").json()
        self.assertEqual(
            resp["fieldsets"][0]["fields"],
            [
                "title",
                "description",
                "sottotitolo",
                "image",
                "image_caption",
                "preview_image",
                "preview_caption",
            ],
        )

    def test_uo_fields_cosa_fa_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/UnitaOrganizzativa").json()
        self.assertEqual(
            resp["fieldsets"][1]["fields"],
            ["competenze"],
        )

    def test_uo_fields_persone_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/UnitaOrganizzativa").json()
        self.assertEqual(
            resp["fieldsets"][2]["fields"],
            ["responsabile_correlato", "personale_correlato"],
        )

    def test_uo_fields_dove_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/UnitaOrganizzativa").json()
        self.assertEqual(
            resp["fieldsets"][3]["fields"],
            [
                "nome_sede",
                "street",
                "zip_code",
                "city",
                "provincia",
                "circoscrizione",
                "country",
                "geolocation",
            ],
        )

    def test_uo_fields_orari_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/UnitaOrganizzativa").json()
        self.assertEqual(resp["fieldsets"][4]["fields"], ["orari"])

    def test_uo_fields_contatti_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/UnitaOrganizzativa").json()
        self.assertEqual(resp["fieldsets"][5]["fields"], ["pdc_correlato"])

    def test_uo_fields_allegati_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/UnitaOrganizzativa").json()
        self.assertEqual(
            resp["fieldsets"][6]["fields"],
            ["documento_correlato"],
        )

    def test_uo_fields_ulteriori_informazioni_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/UnitaOrganizzativa").json()
        self.assertEqual(resp["fieldsets"][7]["fields"], ["ulteriori_informazioni"])


class TestUO(unittest.TestCase):
    """"""

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

    def test_uo_default_children(self):
        uo = api.content.create(
            container=self.portal, type="UnitaOrganizzativa", title="xxx"
        )

        self.assertEqual(uo.keys(), ["allegati"])

    def test_uo_allegati_has_filtered_addable_types(self):
        uo = api.content.create(
            container=self.portal, type="UnitaOrganizzativa", title="xxx"
        )
        allegati = ISelectableConstrainTypes(uo["allegati"])
        self.assertEqual(allegati.getConstrainTypesMode(), 1)
        self.assertEqual(allegati.getLocallyAllowedTypes(), ["File"])



# -*- coding: utf-8 -*-
"""Setup tests for this package."""
from iosanita.contenttypes.testing import INTEGRATION_TESTING
from iosanita.contenttypes.testing import RESTAPI_TESTING
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.restapi.testing import RelativeSession
from Products.CMFPlone.interfaces import ISelectableConstrainTypes

import unittest


class TestEventSchema(unittest.TestCase):
    layer = RESTAPI_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        self.request["LANGUAGE"] = "it"

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)

    def tearDown(self):
        self.api_session.close()

    def test_behaviors_enabled_for_event(self):
        portal_types = api.portal.get_tool(name="portal_types")
        self.assertEqual(
            portal_types["Event"].behaviors,
            (
                "plone.textindexer",
                "plone.eventbasic",
                "plone.eventrecurrence",
                "plone.dublincore",
                "plone.namefromtitle",
                "plone.allowdiscussion",
                "plone.excludefromnavigation",
                "plone.shortname",
                "plone.versioning",
                "plone.locking",
                "plone.constraintypes",
                "volto.preview_image",
                "plone.translatable",
                "kitconcept.seo",
                "iosanita.contenttypes.behavior.a_chi_si_rivolge",
                "collective.taxonomy.generated.a_chi_si_rivolge_tassonomia",
                "iosanita.contenttypes.behavior.ulteriori_informazioni",
                "iosanita.contenttypes.behavior.evento",
                "iosanita.contenttypes.behavior.dove",
                "iosanita.contenttypes.behavior.contatti",
                "iosanita.contenttypes.behavior.ulteriori_informazioni",
                "collective.taxonomy.generated.tipologia_evento",
                "collective.taxonomy.generated.parliamo_di",
            ),
        )

    def test_event_fieldsets(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Event").json()
        self.assertEqual(len(resp["fieldsets"]), 14)
        self.assertEqual(
            [x.get("id") for x in resp["fieldsets"]],
            [
                "default",
                "cosa_e",
                "partecipanti",
                "a_chi_si_rivolge",
                "dove",
                "costi",
                "contatti",
                "ulteriori_informazioni",
                "contenuti_collegati",
                "categorization",
                "dates",
                "ownership",
                "settings",
                "seo",
            ],
        )

    def test_event_required_fields(self):
        resp = self.api_session.get("@types/Event").json()
        self.assertEqual(
            sorted(resp["required"]),
            sorted(
                [
                    "costo",
                    "descrizione_estesa",
                    "end",
                    "pdc_correlato",
                    "start",
                    "tipologia_evento",
                    "title",
                ]
            ),
        )

    def test_event_fields_default_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Event").json()
        self.assertEqual(
            resp["fieldsets"][0]["fields"],
            [
                "title",
                "description",
                "start",
                "end",
                "whole_day",
                "open_end",
                "sync_uid",
                "recurrence",
                "preview_image",
                "preview_caption",
                "tipologia_evento",
                "parliamo_di",
            ],
        )

    def test_event_fields_cosa_e_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Event").json()
        self.assertEqual(resp["fieldsets"][1]["fields"], ["descrizione_estesa"])

    def test_event_fields_partecipanti_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Event").json()
        self.assertEqual(
            resp["fieldsets"][2]["fields"],
            ["persona_correlata", "parteciperanno"],
        )

    def test_event_fields_a_chi_si_rivolge_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Event").json()
        self.assertEqual(
            resp["fieldsets"][3]["fields"],
            ["a_chi_si_rivolge", "a_chi_si_rivolge_tassonomia"],
        )

    def test_event_fields_dove_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Event").json()
        self.assertEqual(
            resp["fieldsets"][4]["fields"],
            [
                "webinar",
                "struttura_correlata",
                "nome_sede",
                "street",
                "zip_code",
                "city",
                "provincia",
                "circoscrizione",
                "country",
                "geolocation",
            ],
        )

    def test_event_fields_costi_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Event").json()
        self.assertEqual(
            resp["fieldsets"][5]["fields"],
            ["costo"],
        )

    def test_event_fields_contatti_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Event").json()
        self.assertEqual(
            resp["fieldsets"][6]["fields"],
            [
                "pdc_correlato",
                "organizzato_da_interno",
                "organizzato_da_esterno",
                "patrocinato_da",
            ],
        )

    def test_event_fields_ulteriori_informazioni_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Event").json()
        self.assertEqual(resp["fieldsets"][7]["fields"], ["ulteriori_informazioni"])

    def test_event_fields_contenuti_collegati_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Event").json()
        self.assertEqual(resp["fieldsets"][8]["fields"], ["eventi_correlati"])


class TestEvent(unittest.TestCase):
    """"""

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

    def test_if_event_has_children_events_is_indexed_as_rassegna(self):
        parent = api.content.create(container=self.portal, type="Event", title="parent")

        brain = api.content.find(UID=parent.UID())[0]

        self.assertFalse(brain.rassegna)

        api.content.create(container=parent, type="Event", title="child")

        brain = api.content.find(UID=parent.UID())[0]

        self.assertTrue(brain.rassegna)

    def test_event_default_children(self):
        event = api.content.create(container=self.portal, type="Event", title="xxx")

        self.assertEqual(
            event.keys(), ["immagini", "video", "sponsor-evento", "allegati"]
        )

    def test_event_immagini_has_filtered_addable_types(self):
        event = api.content.create(container=self.portal, type="Event", title="xxx")
        immagini = ISelectableConstrainTypes(event["immagini"])
        self.assertEqual(immagini.getConstrainTypesMode(), 1)
        self.assertEqual(immagini.getLocallyAllowedTypes(), ["Link", "Image"])

    def test_event_video_has_filtered_addable_types(self):
        event = api.content.create(container=self.portal, type="Event", title="xxx")
        video = ISelectableConstrainTypes(event["video"])
        self.assertEqual(video.getConstrainTypesMode(), 1)
        self.assertEqual(video.getLocallyAllowedTypes(), ["Link"])

    def test_event_sponsor_has_filtered_addable_types(self):
        event = api.content.create(container=self.portal, type="Event", title="xxx")
        sponsor = ISelectableConstrainTypes(event["sponsor-evento"])
        self.assertEqual(sponsor.getConstrainTypesMode(), 1)
        self.assertEqual(sponsor.getLocallyAllowedTypes(), ["Link"])

    def test_event_allegati_has_filtered_addable_types(self):
        event = api.content.create(container=self.portal, type="Event", title="xxx")
        allegati = ISelectableConstrainTypes(event["allegati"])
        self.assertEqual(allegati.getConstrainTypesMode(), 1)
        self.assertEqual(allegati.getLocallyAllowedTypes(), ["File"])



# -*- coding: utf-8 -*-
from collective.volto.blocksfield.field import BlocksField
from iosanita.contenttypes import _
from plone.app.contenttypes.interfaces import INewsItem
from plone.app.dexterity import textindexer
from plone.app.z3cform.widget import RelatedItemsFieldWidget
from plone.autoform import directives as form
from plone.autoform.interfaces import IFormFieldProvider
from plone.supermodel import model
from z3c.relationfield.schema import RelationChoice
from z3c.relationfield.schema import RelationList
from zope import schema
from zope.component import adapter
from zope.interface import implementer
from zope.interface import provider


@provider(IFormFieldProvider)
class INewsAdditionalFields(model.Schema):
    descrizione_estesa = BlocksField(
        title=_("descrizione_estesa_news_label", default="Testo"),
        required=True,
        description=_(
            "descrizione_estesa_news_help",
            default="Testo principale della notizia.",
        ),
    )

    numero_progressivo_cs = schema.TextLine(
        title=_(
            "numero_progressivo_cs_label",
            default="Numero progressivo del comunicato stampa",
        ),
        description=_(
            "numero_progressivo_cs_help",
            default="Se è un comunicato stampa, indicare un'eventuale numero progressivo del comunicato stampa.",
        ),
        required=False,
    )

    persona_correlata = RelationList(
        title=_("persona_correlata_news_label", default="Persone"),
        required=False,
        default=[],
        value_type=RelationChoice(vocabulary="plone.app.vocabularies.Catalog"),
        description=_(
            "persona_correlata_news_help",
            default="Seleziona una serie di Persone dell'ASL citate nella notizia.",
        ),
    )
    struttura_correlata = RelationList(
        title=_("struttura_correlata_news_label", default="Strutture"),
        required=False,
        default=[],
        value_type=RelationChoice(vocabulary="plone.app.vocabularies.Catalog"),
        description=_(
            "struttura_correlata_news_help",
            default="Seleziona una serie di Strutture dell'ASL citate nella notizia.",
        ),
    )

    servizio_correlato = RelationList(
        title=_("servizio_correlato_label", default="Servizi e prestazioni"),
        description=_(
            "servizio_correlato_help",
            default="Elenco dei servizi e delle prestazioni dell'ASL citati nella notizia, con collegamento alle relative pagine foglia servizio. L'elemento è necessario se nella notizia sono citati specifici servizi o prestazioni dell'ASL.",
        ),
        default=[],
        value_type=RelationChoice(vocabulary="plone.app.vocabularies.Catalog"),
        required=False,
    )
    uo_correlata = RelationList(
        title=_("uo_correlata_news_label", default="A cura di"),
        description=_(
            "uo_correlata_news_help",
            default="Unità Organizzativa che ha curato il comunicato.",
        ),
        default=[],
        value_type=RelationChoice(vocabulary="plone.app.vocabularies.Catalog"),
        required=True,
    )
    notizia_correlata = RelationList(
        title=_("notizia_correlata_label", default="Notizie correlate"),
        description=_(
            "notizia_correlata_help",
            default="Elenco di notizie simili o collegate.",
        ),
        default=[],
        value_type=RelationChoice(vocabulary="plone.app.vocabularies.Catalog"),
        required=False,
    )
    # custom widgets
    form.widget(
        "persona_correlata",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "selectableTypes": ["Persona"],
        },
    )
    form.widget(
        "notizia_correlata",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "selectableTypes": ["News Item"],
        },
    )
    form.widget(
        "struttura_correlata",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "selectableTypes": ["Struttura"],
        },
    )
    form.widget(
        "servizio_correlato",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "selectableTypes": ["Servizio"],
        },
    )
    form.widget(
        "uo_correlata",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "selectableTypes": ["UnitaOrganizzativa"],
        },
    )
    model.fieldset(
        "correlati",
        label=_("correlati_label", default="Contenuti collegati"),
        fields=[
            "persona_correlata",
            "struttura_correlata",
            "servizio_correlato",
            "uo_correlata",
            "notizia_correlata",
        ],
    )
    # custom fieldsets and order
    form.order_after(numero_progressivo_cs="description")
    form.order_after(descrizione_estesa="numero_progressivo_cs")

    textindexer.searchable("descrizione_estesa")


@implementer(INewsAdditionalFields)
@adapter(INewsItem)
class NewsAdditionalFields(object):
    """ """

    def __init__(self, context):
        self.context = context



# -*- coding: utf-8 -*-
"""Setup tests for this package."""
from iosanita.contenttypes.testing import INTEGRATION_TESTING
from iosanita.contenttypes.testing import RESTAPI_TESTING
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.restapi.testing import RelativeSession
from Products.CMFPlone.interfaces import ISelectableConstrainTypes

import unittest


class TestBandoSchema(unittest.TestCase):
    layer = RESTAPI_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)

    def tearDown(self):
        self.api_session.close()

    def test_behaviors_enabled_for_bando(self):
        portal_types = api.portal.get_tool(name="portal_types")
        self.assertEqual(
            portal_types["Bando"].behaviors,
            (
                "plone.app.content.interfaces.INameFromTitle",
                "plone.app.dexterity.behaviors.discussion.IAllowDiscussion",
                "plone.app.dexterity.behaviors.exclfromnav.IExcludeFromNavigation",
                "plone.app.dexterity.behaviors.id.IShortName",
                "plone.app.dexterity.behaviors.metadata.IDublinCore",
                "plone.app.relationfield.behavior.IRelatedItems",
                "plone.app.versioningbehavior.behaviors.IVersionable",
                "plone.app.contenttypes.behaviors.tableofcontents.ITableOfContents",
                "plone.app.lockingbehavior.behaviors.ILocking",
                "Products.CMFPlone.interfaces.constrains.ISelectableConstrainTypes",
                "plone.versioning",
                "plone.translatable",
                "kitconcept.seo",
                "volto.preview_image",
                "iosanita.contenttypes.behavior.bando",
                "iosanita.contenttypes.behavior.a_chi_si_rivolge",
                "collective.taxonomy.generated.a_chi_si_rivolge_tassonomia",
                "collective.taxonomy.generated.parliamo_di",
            ),
        )

    def test_bando_fieldsets(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Bando").json()
        self.assertEqual(len(resp["fieldsets"]), 10)
        self.assertEqual(
            [x.get("id") for x in resp["fieldsets"]],
            [
                "default",
                "cosa_e",
                "a_chi_si_rivolge",
                "come_partecipare",
                "modalita_selezione",
                "settings",
                "categorization",
                "dates",
                "ownership",
                "seo",
            ],
        )

    def test_bando_required_fields(self):
        resp = self.api_session.get("@types/Bando").json()
        self.assertEqual(
            sorted(resp["required"]),
            sorted(
                [
                    "title",
                    "destinatari",
                    "tipologia_bando",
                    "descrizione_estesa",
                    "come_partecipare",
                    "modalita_selezione",
                ]
            ),
        )

    def test_bando_fields_default_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Bando").json()
        self.assertEqual(
            resp["fieldsets"][0]["fields"],
            [
                "title",
                "description",
                "riferimenti_bando",
                "apertura_bando",
                "scadenza_domande_bando",
                "chiusura_procedimento_bando",
                "scadenza_bando",
                "ente_bando",
                "destinatari",
                "tipologia_bando",
                "preview_image",
                "preview_caption",
                "note_aggiornamento",
                "parliamo_di",
            ],
        )

    def test_bando_fields_cosa_e_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Bando").json()
        self.assertEqual(resp["fieldsets"][1]["fields"], ["descrizione_estesa"])

    def test_bando_fields_a_chi_si_rivolge_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Bando").json()
        self.assertEqual(
            resp["fieldsets"][2]["fields"],
            ["a_chi_si_rivolge", "a_chi_si_rivolge_tassonomia"],
        )

    def test_bando_fields_come_partecipare_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Bando").json()
        self.assertEqual(
            resp["fieldsets"][3]["fields"],
            [
                "come_partecipare",
            ],
        )

    def test_bando_fields_modalita_selezione_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Bando").json()
        self.assertEqual(
            resp["fieldsets"][4]["fields"],
            [
                "modalita_selezione",
            ],
        )


class TestBando(unittest.TestCase):
    """"""

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

    def test_bando_default_children(self):
        bando = api.content.create(container=self.portal, type="Bando", title="xxx")

        self.assertEqual(
            bando.keys(),
            ["graduatoria", "altri-allegati", "adempimenti-consequenziali"],
        )

        self.assertEqual("Bando Folder Deepening", bando.graduatoria.portal_type)
        self.assertEqual("Bando Folder Deepening", bando["altri-allegati"].portal_type)
        self.assertEqual(
            "Bando Folder Deepening", bando["adempimenti-consequenziali"].portal_type
        )

    def test_bando_graduatoria_has_no_filtered_addable_types(self):
        bando = api.content.create(container=self.portal, type="Bando", title="xxx")
        graduatoria = ISelectableConstrainTypes(bando["graduatoria"])
        self.assertEqual(graduatoria.getConstrainTypesMode(), 0)

    def test_bando_altri_allegati_has_no_filtered_addable_types(self):
        bando = api.content.create(container=self.portal, type="Bando", title="xxx")
        allegati = ISelectableConstrainTypes(bando["altri-allegati"])
        self.assertEqual(allegati.getConstrainTypesMode(), 0)

    def test_bando_adempimenti_has_no_filtered_addable_types(self):
        bando = api.content.create(container=self.portal, type="Bando", title="xxx")
        adempimenti = ISelectableConstrainTypes(bando["adempimenti-consequenziali"])
        self.assertEqual(adempimenti.getConstrainTypesMode(), 0)



# -*- coding: utf-8 -*-
from iosanita.contenttypes.interfaces.step import IStep
from plone.dexterity.content import Container
from zope.interface import implementer


@implementer(IStep)
class Step(Container):
    """ """



from plone.app.testing import applyProfile
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.testing.zope import WSGI_SERVER_FIXTURE
from redturtle.volto.testing import RedturtleVoltoLayer
from zope.configuration import xmlconfig

import collective.address
import collective.geolocationbehavior
import collective.taxonomy
import collective.volto.blocksfield
import collective.volto.enhancedlinks
import collective.z3cform.datagridfield
import iosanita.contenttypes
import iosanita.contenttypes.adapters
import kitconcept.seo
import plone.app.caching
import plone.formwidget.geolocation
import plone.restapi
import redturtle.bandi


class TestLayer(RedturtleVoltoLayer):
    def setUpZope(self, app, configurationContext):
        super().setUpZope(app, configurationContext)
        self.loadZCML(package=collective.taxonomy)
        self.loadZCML(package=collective.volto.blocksfield)
        self.loadZCML(package=collective.volto.enhancedlinks)
        self.loadZCML(package=collective.z3cform.datagridfield)
        self.loadZCML(package=kitconcept.seo)
        self.loadZCML(package=plone.formwidget.geolocation)
        self.loadZCML(package=collective.geolocationbehavior)
        self.loadZCML(package=redturtle.bandi)
        self.loadZCML(package=collective.address)
        self.loadZCML(package=iosanita.contenttypes, context=configurationContext)
        xmlconfig.file(
            "configure.zcml",
            iosanita.contenttypes,
            context=configurationContext,
        )

    def setUpPloneSite(self, portal):
        super().setUpPloneSite(portal)
        applyProfile(portal, "iosanita.contenttypes:default")


FIXTURE = TestLayer()


INTEGRATION_TESTING = IntegrationTesting(
    bases=(FIXTURE,),
    name="IoSanitaContenttypesLayer:IntegrationTesting",
)


FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(FIXTURE,),
    name="IoSanitaContenttypesLayer:FunctionalTesting",
)

RESTAPI_TESTING = FunctionalTesting(
    bases=(FIXTURE, WSGI_SERVER_FIXTURE),
    name="IoSanitaContenttypesLayer:RestAPITesting",
)



# -*- coding: utf-8 -*-
"""Module where all interfaces, events and exceptions live."""
from redturtle.volto.interfaces import IRedturtleVoltoLayer
from zope.interface import Interface


class IIosanitaContenttypesLayer(IRedturtleVoltoLayer):
    """Marker interface that defines a browser layer."""


class IIosanitaContenttypes(Interface):
    """
    Marker interface
    """


class IoSanitaMigrationMarker(Interface):
    """
    Marker interface to disable validation checks and events during migration.
    """


class IoSanitaViewExtraData(Interface):
    """
    Marker interface to get the back references of a given object.
    """



# -*- coding: utf-8 -*-
from collective.volto.blocksfield.field import BlocksField
from iosanita.contenttypes import _
from plone.app.dexterity import textindexer
from plone.app.z3cform.widget import RelatedItemsFieldWidget
from plone.autoform import directives as form
from plone.supermodel import model
from z3c.relationfield.schema import RelationChoice
from z3c.relationfield.schema import RelationList


class IUnitaOrganizzativa(model.Schema):
    """Marker interface for content type UnitaOrganizzativa"""

    competenze = BlocksField(
        title=_("uo_competenze_label", default="Competenze"),
        description=_(
            "uo_competenze_help",
            default="Descrizione dei compiti assegnati a quest unità organizzativa.",
        ),
        required=True,
    )

    responsabile_correlato = RelationList(
        title=_("responsabile_correlato_uo_label", default="Responsabile"),
        description=_(
            "responsabile_correlato_uo_help",
            default="La persona che dirige l'unità organizzativa.",
        ),
        value_type=RelationChoice(vocabulary="plone.app.vocabularies.Catalog"),
        default=[],
        required=False,
    )

    personale_correlato = RelationList(
        title=_("personale_correlato_uo_label", default="Personale"),
        description=_(
            "personale_correlato_uo_help",
            default="Elenco del personale che opera nell'unità organizzativa.",
        ),
        default=[],
        value_type=RelationChoice(vocabulary="plone.app.vocabularies.Catalog"),
        required=False,
    )

    orari = BlocksField(
        title=_("orari_uo_label", default="Orari di apertura"),
        description=_(
            "orari_uo_help",
            default="Indicazione delle fasce orarie in cui è possibile contattare o accedere all'unità organizzativa.",
        ),
        required=True,
    )

    documento_correlato = RelationList(
        title=_("documento_correlato_label", default="Documenti"),
        default=[],
        description=_(
            "documento_correlato_help",
            default="Seleziona dei documenti correlati.",
        ),
        value_type=RelationChoice(vocabulary="plone.app.vocabularies.Catalog"),
        required=False,
    )

    #  custom widgets
    form.widget(
        "documento_correlato",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "selectableTypes": ["Documento"],
        },
    )
    form.widget(
        "personale_correlato",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={"selectableTypes": ["Persona"]},
    )
    form.widget(
        "responsabile_correlato",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "maximumSelectionSize": 1,
            "selectableTypes": ["Persona"],
            # "basePath": "/amministrazione",
        },
    )

    # custom fieldsets and order
    model.fieldset(
        "cosa_fa",
        label=_("cosa_fa_label", default="Competenze"),
        fields=["competenze"],
    )
    model.fieldset(
        "persone_uo",
        label=_("persone_uo_fieldset_label", default="Persone Unità organizzativa"),
        fields=[
            "responsabile_correlato",
            "personale_correlato",
        ],
    )
    model.fieldset(
        "orari",
        label=_("orari_uo_label", default="Orari di apertura"),
        fields=["orari"],
    )

    model.fieldset(
        "documenti",
        label=_("documenti_label", default="Documenti"),
        fields=["documento_correlato"],
    )

    # SearchableText indexers
    textindexer.searchable("competenze")
    textindexer.searchable("orari")



# -*- coding: utf-8 -*-
from iosanita.contenttypes.interfaces.struttura import IStruttura
from plone.dexterity.content import Container
from zope.interface import implementer


@implementer(IStruttura)
class Struttura(Container):
    """ """



# -*- coding: utf-8 -*-

from . import logger
from plone import api


def upgrade(setup_tool=None):
    """ """
    logger.info("Running upgrade (Python): Set dove fields required")

    portal_types = api.portal.get_tool(name="portal_types")

    for ptype in [
        "Struttura",
        "UnitaOrganizzativa",
    ]:
        behaviors = []
        for behavior in portal_types[ptype].behaviors:
            if behavior == "iosanita.contenttypes.behavior.dove":
                behaviors.append("iosanita.contenttypes.behavior.dove_required")
            else:
                behaviors.append(behavior)
        portal_types[ptype].behaviors = tuple(behaviors)

    # enable normale one for persona and event
    behaviors = list(portal_types["Persona"].behaviors)
    if "iosanita.contenttypes.behavior.dove" not in behaviors:
        behaviors.insert(
            behaviors.index("collective.taxonomy.generated.incarico") + 1,
            "iosanita.contenttypes.behavior.dove",
        )
        portal_types["Persona"].behaviors = tuple(behaviors)

    behaviors = list(portal_types["Event"].behaviors)
    if "iosanita.contenttypes.behavior.dove" not in behaviors:
        behaviors.insert(
            behaviors.index("iosanita.contenttypes.behavior.evento") + 1,
            "iosanita.contenttypes.behavior.dove",
        )
        portal_types["Event"].behaviors = tuple(behaviors)



# -*- coding: utf-8 -*-
from collective.volto.blocksfield.field import BlocksField
from iosanita.contenttypes import _
from iosanita.contenttypes.interfaces import IIosanitaContenttypes
from plone.app.dexterity import textindexer
from plone.app.z3cform.widget import RelatedItemsFieldWidget
from plone.autoform import directives as form
from plone.supermodel import model
from z3c.relationfield.schema import RelationChoice
from z3c.relationfield.schema import RelationList
from zope import schema


class IStep(model.Schema, IIosanitaContenttypes):
    """Marker interface for content type Step"""

    title = schema.TextLine(title=_("label_title", default="Titolo"), required=True)

    testo = BlocksField(
        title=_("testo_label", default="Testo"),
        description=_(
            "testo_help",
            default="Descrizione del passo della procedura.",
        ),
        required=False,
    )
    uo_correlata = RelationList(
        title=_("uo_correlata_step_label", default="Dove"),
        description=_(
            "uo_correlata_step_help",
            default="Seleziona una Unità organizzativa.",
        ),
        default=[],
        value_type=RelationChoice(vocabulary="plone.app.vocabularies.Catalog"),
        required=False,
    )

    # questo lo mettiamo qui perché é l'unico non obbligatorio
    pdc_correlato = RelationList(
        title=_(
            "pdc_correlato_label",
            default="Punti di contatto",
        ),
        description=_(
            "pdc_correlato_help",
            default="Seleziona una lista di punti di contatto.",
        ),
        required=False,
        default=[],
        value_type=RelationChoice(
            vocabulary="plone.app.vocabularies.Catalog",
        ),
    )

    # custom widgets
    form.widget(
        "pdc_correlato",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "selectableTypes": ["PuntoDiContatto"],
        },
    )

    form.widget(
        "uo_correlata",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "selectableTypes": ["UnitaOrganizzativa"],
        },
    )

    # custom fieldsets
    model.fieldset(
        "contatti",
        label=_("contatti_label", default="Contatti"),
        fields=["pdc_correlato"],
    )

    # SearchableText fields
    textindexer.searchable("title")
    textindexer.searchable("testo")



# -*- coding: utf-8 -*-
from Acquisition import aq_inner
from collective.volto.enhancedlinks.interfaces import IEnhancedLinksEnabled
from iosanita.contenttypes.interfaces import IIosanitaContenttypesLayer
from plone.base.utils import human_readable_size
from plone.dexterity.interfaces import IDexterityContent
from plone.namedfile.interfaces import INamedFileField
from plone.restapi.serializer.converters import json_compatible
from plone.restapi.serializer.dxfields import DefaultFieldSerializer
from zope.component import adapter


@adapter(INamedFileField, IDexterityContent, IIosanitaContenttypesLayer)
class FileFieldViewModeSerializer(DefaultFieldSerializer):
    """
    Ovveride the basic DX serializer to:
        - handle the visualize file functionality
        - add getObjSize info
    """

    def __call__(self):
        namedfile = self.field.get(self.context)
        if namedfile is None:
            return

        url = "/".join(
            (
                self.context.absolute_url(),
                self.get_file_view_mode(namedfile.contentType),
                self.field.__name__,
            )
        )
        size = namedfile.getSize()
        result = {
            "filename": namedfile.filename,
            "content-type": namedfile.contentType,
            "size": size,
            "download": url,
        }
        if IEnhancedLinksEnabled.providedBy(self.context):
            result.update(
                {
                    "getObjSize": human_readable_size(size),
                    "enhanced_links_enabled": True,
                }
            )

        return json_compatible(result)

    def get_file_view_mode(self, content_type):
        """Pdf view depends on the anteprima_file property in thq aq_chain"""
        if self.context and "pdf" in content_type:
            if getattr(aq_inner(self.context), "anteprima_file", None):
                return "@@display-file"

        return "@@download"



# -*- coding: utf-8 -*-
"""Setup tests for this package."""
from iosanita.contenttypes.testing import INTEGRATION_TESTING
from iosanita.contenttypes.testing import RESTAPI_TESTING
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.restapi.testing import RelativeSession
from Products.CMFPlone.interfaces import ISelectableConstrainTypes

import unittest


class TestServizioSchema(unittest.TestCase):
    layer = RESTAPI_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)

    def tearDown(self):
        self.api_session.close()

    def test_behaviors_enabled_for_servizio(self):
        portal_types = api.portal.get_tool(name="portal_types")
        self.assertEqual(
            portal_types["Servizio"].behaviors,
            (
                "plone.namefromtitle",
                "plone.allowdiscussion",
                "plone.excludefromnavigation",
                "plone.shortname",
                "plone.ownership",
                "plone.publication",
                "plone.categorization",
                "iosanita.basic",
                "iosanita.contenttypes.behavior.sottotitolo",
                "plone.locking",
                "plone.constraintypes",
                "plone.leadimage",
                "volto.preview_image",
                "plone.relateditems",
                "plone.textindexer",
                "plone.translatable",
                "kitconcept.seo",
                "plone.versioning",
                "collective.taxonomy.generated.tipologia_servizio",
                "iosanita.contenttypes.behavior.contatti",
                "iosanita.contenttypes.behavior.a_chi_si_rivolge",
                "collective.taxonomy.generated.a_chi_si_rivolge_tassonomia",
                "collective.taxonomy.generated.parliamo_di",
                "iosanita.contenttypes.behavior.ulteriori_informazioni",
            ),
        )

    def test_servizio_fieldsets(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Servizio").json()
        self.assertEqual(len(resp["fieldsets"]), 19)
        self.assertEqual(
            [x.get("id") for x in resp["fieldsets"]],
            [
                "default",
                "cosa_serve",
                "accedi_al_servizio",
                "tempi_attesa",
                "costi",
                "dove",
                "orari",
                "contatti",
                "cosa_e",
                "a_chi_si_rivolge",
                "procedure_collegate_esito",
                "responsabili",
                "ulteriori_informazioni",
                "contenuti_collegati",
                "settings",
                "ownership",
                "dates",
                "categorization",
                "seo",
            ],
        )

    def test_servizio_required_fields(self):
        resp = self.api_session.get("@types/Servizio").json()
        self.assertEqual(
            sorted(resp["required"]),
            sorted(
                [
                    "title",
                    "description",
                    "cosa_serve",
                    "come_accedere",
                    "struttura_correlata",
                    "orari",
                    "pdc_correlato",
                ]
            ),
        )

    def test_servizio_fields_default_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Servizio").json()
        self.assertEqual(
            resp["fieldsets"][0]["fields"],
            [
                "title",
                "description",
                "servizio_attivo",
                "sottotitolo",
                "image",
                "image_caption",
                "preview_image",
                "preview_caption",
                "tipologia_servizio",
                "parliamo_di",
            ],
        )

    def test_servizio_fields_cosa_serve_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Servizio").json()
        self.assertEqual(
            resp["fieldsets"][1]["fields"],
            ["cosa_serve"],
        )

    def test_servizio_fields_accedi_al_servizio_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Servizio").json()
        self.assertEqual(
            resp["fieldsets"][2]["fields"],
            ["come_accedere", "prenota_online_link", "prenota_online_label"],
        )

    def test_servizio_fields_tempi_attesa_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Servizio").json()
        self.assertEqual(
            resp["fieldsets"][3]["fields"],
            ["tempi_attesa"],
        )

    def test_servizio_fields_costi_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Servizio").json()
        self.assertEqual(
            resp["fieldsets"][4]["fields"],
            ["costi"],
        )

    def test_servizio_fields_struttura_correlata_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Servizio").json()
        self.assertEqual(resp["fieldsets"][5]["fields"], ["struttura_correlata"])

    def test_servizio_fields_orari_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Servizio").json()
        self.assertEqual(resp["fieldsets"][6]["fields"], ["orari"])

    def test_servizio_fields_contatti_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Servizio").json()
        self.assertEqual(resp["fieldsets"][7]["fields"], ["pdc_correlato"])

    def test_servizio_fields_cosa_e_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Servizio").json()
        self.assertEqual(resp["fieldsets"][8]["fields"], ["descrizione_estesa"])

    def test_servizio_fields_a_chi_si_rivolge_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Servizio").json()
        self.assertEqual(
            resp["fieldsets"][9]["fields"],
            ["a_chi_si_rivolge", "a_chi_si_rivolge_tassonomia"],
        )

    def test_servizio_fields_esito_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Servizio").json()
        self.assertEqual(resp["fieldsets"][10]["fields"], ["procedure_collegate_esito"])

    def test_servizio_fields_responsabili_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Servizio").json()
        self.assertEqual(
            resp["fieldsets"][11]["fields"],
            ["uo_correlata", "responsabile_correlato"],
        )

    def test_servizio_fields_ulteriori_informazioni_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Servizio").json()
        self.assertEqual(resp["fieldsets"][12]["fields"], ["ulteriori_informazioni"])

    def test_servizio_fields_contenuti_collegati_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Servizio").json()
        self.assertEqual(resp["fieldsets"][13]["fields"], ["servizio_correlato"])


class TestServizio(unittest.TestCase):
    """"""

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

    def test_servizio_default_children(self):
        servizio = api.content.create(
            container=self.portal, type="Servizio", title="xxx"
        )

        self.assertEqual(servizio.keys(), ["modulistica", "allegati"])

    def test_servizio_allegati_has_filtered_addable_types(self):
        servizio = api.content.create(
            container=self.portal, type="Servizio", title="xxx"
        )
        allegati = ISelectableConstrainTypes(servizio["allegati"])
        self.assertEqual(allegati.getConstrainTypesMode(), 1)
        self.assertEqual(allegati.getLocallyAllowedTypes(), ["File"])



# -*- coding: utf-8 -*-

from . import logger
from iosanita.contenttypes.events.common import SUBFOLDERS_MAPPING
from plone import api


# from plone import api


def upgrade(setup_tool=None):
    """ """
    logger.info("Running upgrade (Python): Exclude from nav service folders")

    for portal_type, mapping in SUBFOLDERS_MAPPING.items():
        children_ids = [x["id"] for x in mapping["content"]]

        container_brains = api.content.find(portal_type=portal_type)
        tot = len(container_brains)
        logger.info(f"Fixing {tot} {portal_type}")
        for brain in container_brains:
            item = brain.getObject()
            for child in item.listFolderContents():
                if child.getId() in children_ids and child.portal_type in [
                    "Document",
                    "Folder",
                ]:
                    child.exclude_from_nav = True
                    child.reindexObject(idxs=["exclude_from_nav"])



# -*- coding: utf-8 -*-
"""Setup tests for this package."""
from iosanita.contenttypes.testing import RESTAPI_TESTING
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.namedfile.file import NamedBlobFile
from plone.restapi.testing import RelativeSession
from transaction import commit
from z3c.relationfield import RelationValue
from zope.component import getUtility
from zope.intid.interfaces import IIntIds

import os
import unittest


class TestExpander(unittest.TestCase):
    """"""

    layer = RESTAPI_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)

    def tearDown(self):
        self.api_session.close()

    def test_expander_always_expanded(self):
        servizio = api.content.create(
            container=self.portal, type="Servizio", title="Servizio"
        )
        commit()
        resp = self.api_session.get(f"{servizio.absolute_url()}").json()
        self.assertIn("back-references", resp["@components"]["view-extra-data"])


class TestBackReferences(unittest.TestCase):
    """"""

    layer = RESTAPI_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)

    def tearDown(self):
        self.api_session.close()

    def test_expander_called_on_generic_content_does_not_return_anything(self):
        link = api.content.create(container=self.portal, type="Link", title="Link")
        commit()
        resp = self.api_session.get(f"{link.absolute_url()}/@view-extra-data").json()

        self.assertNotIn("back-references", resp)

    def test_expander_on_registered_ct_return_nothing_if_no_relations_are_set(self):
        servizio = api.content.create(
            container=self.portal, type="Servizio", title="Servizio"
        )
        commit()
        resp = self.api_session.get(
            f"{servizio.absolute_url()}/@view-extra-data"
        ).json()

        back_references = resp["back-references"]
        self.assertEqual([], list(back_references.keys()))

    def test_expander_return_related_news_sorted_by_Date(self):
        """
        Date metadata is modifified date for contents not yet published, or
        effective date for published contents
        """
        servizio = api.content.create(
            container=self.portal, type="Servizio", title="Servizio"
        )
        intids = getUtility(IIntIds)
        news1 = api.content.create(
            container=self.portal,
            type="News Item",
            title="News 1",
            servizio_correlato=[RelationValue(intids.getId(servizio))],
        )
        news2 = api.content.create(
            container=self.portal,
            type="News Item",
            title="News 2",
            servizio_correlato=[RelationValue(intids.getId(servizio))],
        )
        news3 = api.content.create(
            container=self.portal,
            type="News Item",
            title="News 3",
            servizio_correlato=[RelationValue(intids.getId(servizio))],
        )

        api.content.transition(obj=news2, transition="publish")
        api.content.transition(obj=news3, transition="publish")
        api.content.transition(obj=news1, transition="publish")

        news2.setEffectiveDate(news2.effective() + 1)
        news3.setEffectiveDate(news3.effective() + 2)
        news1.setEffectiveDate(news1.effective() + 3)

        news1.reindexObject()
        news2.reindexObject()
        news3.reindexObject()

        commit()

        resp = self.api_session.get(
            f"{servizio.absolute_url()}/@view-extra-data"
        ).json()
        back_references = resp["back-references"]

        self.assertEqual(len(back_references["News Item"]), 3)
        self.assertEqual(back_references["News Item"][0]["@id"], news1.absolute_url())
        self.assertEqual(back_references["News Item"][1]["@id"], news3.absolute_url())
        self.assertEqual(back_references["News Item"][2]["@id"], news2.absolute_url())

    # SERVIZIO

    def test_servizio_expander_return_related_contents_splitted_by_portal_type(self):
        servizio = api.content.create(
            container=self.portal, type="Servizio", title="Servizio"
        )
        intids = getUtility(IIntIds)
        documento = api.content.create(
            container=self.portal,
            type="Documento",
            title="documento",
            servizio_procedura_riferimento=[RelationValue(intids.getId(servizio))],
        )

        servizio2 = api.content.create(
            container=self.portal,
            type="Servizio",
            title="servizio 2",
            servizio_correlato=[RelationValue(intids.getId(servizio))],
        )

        commit()

        resp = self.api_session.get(
            f"{servizio.absolute_url()}/@view-extra-data"
        ).json()

        back_references = resp["back-references"]
        self.assertEqual(len(back_references["Documento"]), 1)
        self.assertEqual(len(back_references["Servizio"]), 1)
        self.assertEqual(
            back_references["Documento"][0]["@id"],
            documento.absolute_url(),
        )
        self.assertEqual(
            back_references["Servizio"][0]["@id"],
            servizio2.absolute_url(),
        )

    def test_struttura_expander_return_related_contents_splitted_by_portal_type(self):
        struttura = api.content.create(
            container=self.portal, type="Struttura", title="Struttura"
        )
        intids = getUtility(IIntIds)
        news = api.content.create(
            container=self.portal,
            type="News Item",
            title="News",
            struttura_correlata=[RelationValue(intids.getId(struttura))],
        )
        event = api.content.create(
            container=self.portal,
            type="Event",
            title="Event",
            struttura_correlata=[RelationValue(intids.getId(struttura))],
        )
        service = api.content.create(
            container=self.portal,
            type="Servizio",
            title="Servizio",
            struttura_correlata=[RelationValue(intids.getId(struttura))],
        )

        commit()

        resp = self.api_session.get(
            f"{struttura.absolute_url()}/@view-extra-data"
        ).json()
        back_references = resp["back-references"]

        self.assertEqual(
            list(back_references.keys()),
            ["Event", "News Item", "Servizio"],
        )

        self.assertEqual(len(back_references["Event"]), 1)
        self.assertEqual(len(back_references["News Item"]), 1)
        self.assertEqual(len(back_references["Servizio"]), 1)

        self.assertEqual(back_references["Event"][0]["@id"], event.absolute_url())
        self.assertEqual(back_references["News Item"][0]["@id"], news.absolute_url())
        self.assertEqual(back_references["Servizio"][0]["@id"], service.absolute_url())

    def test_uo_expander_return_related_contents_splitted_by_portal_type(self):
        uo = api.content.create(
            container=self.portal, type="UnitaOrganizzativa", title="uo"
        )
        intids = getUtility(IIntIds)

        news = api.content.create(
            container=self.portal,
            type="News Item",
            title="News",
            uo_correlata=[RelationValue(intids.getId(uo))],
        )
        documento = api.content.create(
            container=self.portal,
            type="Documento",
            title="Documento",
            uo_correlata=[RelationValue(intids.getId(uo))],
        )
        pdc = api.content.create(
            container=self.portal,
            type="PuntoDiContatto",
            title="pdc",
            uo_correlata=[RelationValue(intids.getId(uo))],
        )
        servizio = api.content.create(
            container=self.portal,
            type="Servizio",
            title="Servizio",
            uo_correlata=[RelationValue(intids.getId(uo))],
        )
        struttura = api.content.create(
            container=self.portal,
            type="Struttura",
            title="struttura",
            uo_correlata=[RelationValue(intids.getId(uo))],
        )

        commit()

        resp = self.api_session.get(f"{uo.absolute_url()}/@view-extra-data").json()
        back_references = resp["back-references"]

        self.assertEqual(
            list(back_references.keys()),
            [
                "Documento",
                "News Item",
                "PuntoDiContatto",
                "Servizio",
                "Struttura",
            ],
        )

        self.assertEqual(len(back_references["Documento"]), 1)
        self.assertEqual(len(back_references["News Item"]), 1)
        self.assertEqual(len(back_references["PuntoDiContatto"]), 1)
        self.assertEqual(len(back_references["Servizio"]), 1)
        self.assertEqual(len(back_references["Struttura"]), 1)

        self.assertEqual(
            back_references["Documento"][0]["@id"], documento.absolute_url()
        )
        self.assertEqual(back_references["News Item"][0]["@id"], news.absolute_url())
        self.assertEqual(back_references["Servizio"][0]["@id"], servizio.absolute_url())
        self.assertEqual(
            back_references["PuntoDiContatto"][0]["@id"], pdc.absolute_url()
        )
        self.assertEqual(
            back_references["Struttura"][0]["@id"], struttura.absolute_url()
        )

    def test_persona_expander_return_related_contents_splitted_by_portal_type(self):
        persona = api.content.create(
            container=self.portal, type="Persona", title="persona"
        )
        intids = getUtility(IIntIds)

        news = api.content.create(
            container=self.portal,
            type="News Item",
            title="News",
            persona_correlata=[RelationValue(intids.getId(persona))],
        )
        evento = api.content.create(
            container=self.portal,
            type="Event",
            title="event",
            persona_correlata=[RelationValue(intids.getId(persona))],
        )

        commit()

        resp = self.api_session.get(f"{persona.absolute_url()}/@view-extra-data").json()
        back_references = resp["back-references"]

        self.assertEqual(
            list(back_references.keys()),
            ["Event", "News Item", "personale", "responsabile"],
        )

        self.assertEqual(len(back_references["News Item"]), 1)
        self.assertEqual(len(back_references["Event"]), 1)

        self.assertEqual(back_references["News Item"][0]["@id"], news.absolute_url())
        self.assertEqual(back_references["Event"][0]["@id"], evento.absolute_url())

    def test_persona_expander_return_responsabile_and_personale_splitted_by_type(self):
        persona = api.content.create(
            container=self.portal, type="Persona", title="persona"
        )
        intids = getUtility(IIntIds)

        uo = api.content.create(
            container=self.portal,
            type="UnitaOrganizzativa",
            title="uo",
            responsabile_correlato=[RelationValue(intids.getId(persona))],
            personale_correlato=[RelationValue(intids.getId(persona))],
        )
        servizio = api.content.create(
            container=self.portal,
            type="Servizio",
            title="Servizio",
            responsabile_correlato=[RelationValue(intids.getId(persona))],
        )
        struttura = api.content.create(
            container=self.portal,
            type="Struttura",
            title="struttura",
            responsabile_correlato=[RelationValue(intids.getId(persona))],
        )

        commit()

        resp = self.api_session.get(f"{persona.absolute_url()}/@view-extra-data").json()
        back_references = resp["back-references"]

        self.assertEqual(
            list(back_references.keys()),
            [
                "personale",
                "responsabile",
            ],
        )

        responsabile = back_references["responsabile"]
        personale = back_references["personale"]

        self.assertEqual(
            list(responsabile.keys()),
            ["Servizio", "Struttura", "UnitaOrganizzativa"],
        )
        self.assertEqual(
            list(personale.keys()),
            ["UnitaOrganizzativa"],
        )

        self.assertEqual(len(responsabile["UnitaOrganizzativa"]), 1)
        self.assertEqual(len(responsabile["Servizio"]), 1)
        self.assertEqual(len(responsabile["Struttura"]), 1)
        self.assertEqual(responsabile["Servizio"][0]["@id"], servizio.absolute_url())
        self.assertEqual(
            responsabile["UnitaOrganizzativa"][0]["@id"], uo.absolute_url()
        )
        self.assertEqual(responsabile["Struttura"][0]["@id"], struttura.absolute_url())

        self.assertEqual(len(personale["UnitaOrganizzativa"]), 1)
        self.assertEqual(personale["UnitaOrganizzativa"][0]["@id"], uo.absolute_url())


class TestBandoExtraData(unittest.TestCase):
    """"""

    layer = RESTAPI_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)

        self.bando = api.content.create(
            container=self.portal, type="Bando", title="Bando"
        )

        commit()

    def tearDown(self):
        self.api_session.close()

    def test_stato_bando_info_in_expander(self):
        resp = self.api_session.get(
            f"{self.bando.absolute_url()}/@view-extra-data"
        ).json()

        self.assertIn("stato_bando", resp)

    def test_approfondimenti_info_in_expander(self):
        resp = self.api_session.get(
            f"{self.bando.absolute_url()}/@view-extra-data"
        ).json()

        self.assertIn("approfondimenti", resp)

    def test_approfondimenti_data(self):
        # empty if no files in some bando children
        resp = self.api_session.get(
            f"{self.bando.absolute_url()}/@view-extra-data"
        ).json()
        self.assertEqual(resp["approfondimenti"], [])

        # now fill one folder
        filename = os.path.join(os.path.dirname(__file__), "example.txt")
        api.content.create(
            container=self.bando["altri-allegati"],
            type="File",
            title="attachment",
            file=NamedBlobFile(
                data=open(filename, "rb").read(),
                filename="example.txt",
                contentType="text/plain",
            ),
        )
        commit()

        resp = self.api_session.get(
            f"{self.bando.absolute_url()}/@view-extra-data"
        ).json()

        self.assertEqual(len(resp["approfondimenti"]), 1)
        self.assertEqual(resp["approfondimenti"][0]["title"], "Altri allegati")
        self.assertEqual(len(resp["approfondimenti"][0]["items"]), 1)
        self.assertEqual(resp["approfondimenti"][0]["items"][0]["title"], "attachment")



from iosanita.contenttypes.interfaces import IIosanitaContenttypesLayer
from plone.dexterity.interfaces import IDexterityContent
from plone.restapi.interfaces import IFieldSerializer
from plone.restapi.serializer.relationfield import (
    RelationListFieldSerializer as BaseSerializer,
)
from z3c.relationfield.interfaces import IRelationList
from zope.component import adapter
from zope.interface import implementer


@adapter(IRelationList, IDexterityContent, IIosanitaContenttypesLayer)
@implementer(IFieldSerializer)
class RelationListFieldSerializer(BaseSerializer):
    def __call__(self):
        """
        Do not return broken relations
        """
        data = super().__call__()
        return [x for x in data if x is not None]



# -*- coding: utf-8 -*-
from uuid import uuid4

import logging


logger = logging.getLogger(__name__)


def create_default_blocks(context):
    title_uuid = str(uuid4())
    context.blocks = {title_uuid: {"@type": "title"}}
    context.blocks_layout = {"items": [title_uuid]}



# -*- coding: utf-8 -*-
from iosanita.contenttypes import _
from plone.app.contenttypes.interfaces import IDocument
from plone.app.z3cform.widget import RelatedItemsFieldWidget
from plone.autoform import directives as form
from plone.autoform.interfaces import IFormFieldProvider
from plone.dexterity.interfaces import IDexterityContent
from plone.supermodel import model
from z3c.relationfield.schema import RelationChoice
from z3c.relationfield.schema import RelationList
from zope.component import adapter
from zope.interface import implementer
from zope.interface import provider


class IArgomentiSchema(model.Schema):
    """Marker interface for Argomenti"""

    correlato_in_evidenza = RelationList(
        title=_("correlato_in_evidenza_label", default="Correlato in evidenza"),
        description=_(
            "correlato_in_evidenza_help",
            default="Seleziona un correlato da mettere in evidenza per questo"
            " contenuto.",
        ),
        value_type=RelationChoice(
            title=_("Correlato in evidenza"),
            vocabulary="plone.app.vocabularies.Catalog",
        ),
        required=False,
        default=[],
    )

    form.widget(
        "correlato_in_evidenza",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={"maximumSelectionSize": 1},
    )


@provider(IFormFieldProvider)
class IArgomenti(IArgomentiSchema):
    """ """

    model.fieldset(
        "correlati",
        label=_("correlati_label", default="Contenuti collegati"),
        fields=["correlato_in_evidenza"],
    )
    form.order_after(correlato_in_evidenza="IRelatedItems.relatedItems")


@provider(IFormFieldProvider)
class IArgomentiDocument(IArgomentiSchema):
    """ """

    model.fieldset(
        "correlati",
        label=_("correlati_label", default="Contenuti collegati"),
        fields=["correlato_in_evidenza"],
    )

    form.order_after(correlato_in_evidenza="IRelatedItems.relatedItems")


@implementer(IArgomenti)
@adapter(IDexterityContent)
class Argomenti(object):
    """"""

    def __init__(self, context):
        self.context = context


@implementer(IArgomentiDocument)
@adapter(IDocument)
class ArgomentiDocument(Argomenti):
    """"""

    def __init__(self, context):
        self.context = context



# -*- coding: utf-8 -*-
"""Setup tests for this package."""
from iosanita.contenttypes.testing import INTEGRATION_TESTING
from iosanita.contenttypes.testing import RESTAPI_TESTING
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.dexterity.utils import createContentInContainer
from plone.restapi.testing import RelativeSession
from Products.CMFPlone.interfaces import ISelectableConstrainTypes

import unittest


class TestPersonaSchema(unittest.TestCase):
    layer = RESTAPI_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)

    def tearDown(self):
        self.api_session.close()

    def test_behaviors_enabled_for_persona(self):
        portal_types = api.portal.get_tool(name="portal_types")
        self.assertEqual(
            portal_types["Persona"].behaviors,
            (
                "plone.namefromtitle",
                "plone.allowdiscussion",
                "plone.excludefromnavigation",
                "plone.shortname",
                "plone.ownership",
                "plone.publication",
                "plone.relateditems",
                "plone.categorization",
                "plone.locking",
                "plone.textindexer",
                "plone.translatable",
                "kitconcept.seo",
                "plone.versioning",
                "plone.constraintypes",
                "collective.taxonomy.generated.incarico",
                "iosanita.contenttypes.behavior.dove",
                "iosanita.contenttypes.behavior.contatti",
                "iosanita.contenttypes.behavior.ulteriori_informazioni",
            ),
        )

    def test_persona_fieldsets(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Persona").json()
        self.assertEqual(len(resp["fieldsets"]), 13)
        self.assertEqual(
            [x.get("id") for x in resp["fieldsets"]],
            [
                "default",
                "incarichi",
                "competenze",
                "dove",
                "orari_ricevimento",
                "contatti",
                "biografia",
                "ulteriori_informazioni",
                "settings",
                "ownership",
                "dates",
                "categorization",
                "seo",
            ],
        )

    def test_persona_required_fields(self):
        resp = self.api_session.get("@types/Persona").json()
        self.assertEqual(
            sorted(resp["required"]),
            sorted(
                [
                    "nome",
                    "cognome",
                    "incarico",
                    "competenze",
                    "pdc_correlato",
                ]
            ),
        )

    def test_persona_fields_default_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Persona").json()
        self.assertEqual(
            resp["fieldsets"][0]["fields"],
            [
                "cognome",
                "nome",
                "titolo_persona",
                "description",
                "image",
            ],
        )

    def test_persona_fields_incarichi_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Persona").json()
        self.assertEqual(
            resp["fieldsets"][1]["fields"], ["incarico", "altri_incarichi"]
        )

    def test_persona_fields_competenze_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Persona").json()
        self.assertEqual(
            resp["fieldsets"][2]["fields"],
            ["competenze"],
        )

    def test_persona_fields_dove_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Persona").json()
        self.assertEqual(
            resp["fieldsets"][3]["fields"],
            [
                "struttura_ricevimento",
                "struttura_in_cui_opera",
                "nome_sede",
                "street",
                "zip_code",
                "city",
                "provincia",
                "circoscrizione",
                "country",
                "geolocation",
            ],
        )

    def test_persona_fields_orari_ricevimento_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Persona").json()
        self.assertEqual(
            resp["fieldsets"][4]["fields"],
            ["orari_ricevimento"],
        )

    def test_persona_fields_contatti_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Persona").json()
        self.assertEqual(resp["fieldsets"][5]["fields"], ["pdc_correlato"])

    def test_persona_fields_biografia_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Persona").json()
        self.assertEqual(resp["fieldsets"][6]["fields"], ["biografia"])

    def test_persona_fields_ulteriori_informazioni_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Persona").json()
        self.assertEqual(resp["fieldsets"][7]["fields"], ["ulteriori_informazioni"])


class TestPersona(unittest.TestCase):
    """"""

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

    def test_persona_title_composed(self):
        createContentInContainer(self.portal, "Persona", nome="John", cognome="Doe")
        self.assertIn("doe-john", self.portal.keys())
        persona = self.portal["doe-john"]
        self.assertEqual(persona.title, "Doe John")

    def test_persona_title_composed_also_with_titolo(self):
        createContentInContainer(
            self.portal, "Persona", nome="John", cognome="Doe", titolo_persona="dr."
        )
        self.assertIn("dr-doe-john", self.portal.keys())
        persona = self.portal["dr-doe-john"]
        self.assertEqual(persona.title, "dr. Doe John")

    def test_persona_default_children(self):
        createContentInContainer(
            self.portal, "Persona", nome="John", cognome="Doe", titolo_persona="dr."
        )
        persona = self.portal["dr-doe-john"]

        self.assertEqual(
            persona.keys(), ["curriculum-vitae", "immagini", "video", "allegati"]
        )

    def test_persona_immagini_has_filtered_addable_types(self):
        persona = api.content.create(container=self.portal, type="Persona", title="xxx")
        immagini = ISelectableConstrainTypes(persona["immagini"])
        self.assertEqual(immagini.getConstrainTypesMode(), 1)
        self.assertEqual(immagini.getLocallyAllowedTypes(), ["Link", "Image"])

    def test_persona_video_has_filtered_addable_types(self):
        persona = api.content.create(container=self.portal, type="Persona", title="xxx")
        video = ISelectableConstrainTypes(persona["video"])
        self.assertEqual(video.getConstrainTypesMode(), 1)
        self.assertEqual(video.getLocallyAllowedTypes(), ["Link"])

    def test_persona_curriculum_has_filtered_addable_types(self):
        persona = api.content.create(container=self.portal, type="Persona", title="xxx")
        curriculum = ISelectableConstrainTypes(persona["curriculum-vitae"])
        self.assertEqual(curriculum.getConstrainTypesMode(), 1)
        self.assertEqual(curriculum.getLocallyAllowedTypes(), ["File"])

    def test_persona_allegati_has_filtered_addable_types(self):
        persona = api.content.create(container=self.portal, type="Persona", title="xxx")
        allegati = ISelectableConstrainTypes(persona["allegati"])
        self.assertEqual(allegati.getConstrainTypesMode(), 1)
        self.assertEqual(allegati.getLocallyAllowedTypes(), ["File"])



# -*- coding: utf-8 -*-
"""Setup tests for this package."""
from iosanita.contenttypes.testing import RESTAPI_TESTING
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.restapi.interfaces import ISerializeToJsonSummary
from plone.restapi.testing import RelativeSession
from transaction import commit
from zope.component import getMultiAdapter

import unittest


class TestSerializerSummary(unittest.TestCase):
    """"""

    layer = RESTAPI_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]

        self.request["LANGUAGE"] = "it"

        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)

    def test_summary_serializer_always_return_parliamo_di_metadata(self):
        """ """
        struttura = api.content.create(
            container=self.portal,
            type="Struttura",
            title="Test servizio",
            parliamo_di=["ticket-ed-esenzioni", "igiene-pubblica"],
        )
        commit()

        resp = self.api_session.get(f"@search?UID={struttura.UID()}").json()
        self.assertEqual(resp["items_total"], 1)
        self.assertIn("parliamo_di_metadata", resp["items"][0])
        self.assertEqual(
            resp["items"][0]["parliamo_di_metadata"],
            [
                {"title": "Ticket ed esenzioni", "token": "ticket-ed-esenzioni"},
                {"title": "Igiene Pubblica", "token": "igiene-pubblica"},
            ],
        )

    def test_summary_serializer_always_return_a_chi_si_rivolge_tassonomia_metadata(
        self,
    ):
        """ """
        struttura = api.content.create(
            container=self.portal,
            type="Struttura",
            title="Test servizio",
            a_chi_si_rivolge_tassonomia=[
                "farmacie",
                "imprese",
            ],
        )

        commit()
        resp = self.api_session.get(f"@search?UID={struttura.UID()}").json()

        self.assertEqual(resp["items_total"], 1)
        self.assertIn("a_chi_si_rivolge_tassonomia_metadata", resp["items"][0])
        self.assertEqual(
            resp["items"][0]["a_chi_si_rivolge_tassonomia_metadata"],
            [
                {"title": "Farmacie", "token": "farmacie"},
                {"title": "Imprese", "token": "imprese"},
            ],
        )

    def test_summary_serializer_always_return_id_metadata(
        self,
    ):
        """ """
        struttura = api.content.create(
            container=self.portal,
            type="Struttura",
            title="Test servizio",
            a_chi_si_rivolge_tassonomia=[
                "farmacie",
                "imprese",
            ],
        )

        commit()
        resp = self.api_session.get(f"@search?UID={struttura.UID()}").json()

        self.assertEqual(resp["items_total"], 1)
        self.assertIn("id", resp["items"][0])
        self.assertEqual(
            resp["items"][0]["id"],
            "test-servizio",
        )

    def test_summary_serializer_always_return_tipologia_notizia_metadata(
        self,
    ):
        """ """
        struttura = api.content.create(
            container=self.portal,
            type="Struttura",
            title="Test servizio",
            a_chi_si_rivolge_tassonomia=[
                "farmacie",
                "imprese",
            ],
        )

        news = api.content.create(
            container=self.portal,
            type="News Item",
            title="Test news",
            tipologia_notizia=["notizia"],
        )

        commit()
        resp = self.api_session.get(f"@search?UID={struttura.UID()}").json()

        self.assertEqual(resp["items_total"], 1)
        self.assertIn("tipologia_notizia", resp["items"][0])
        self.assertEqual(resp["items"][0]["tipologia_notizia"], [])

        resp = self.api_session.get(f"@search?UID={news.UID()}").json()

        self.assertEqual(resp["items_total"], 1)
        self.assertIn("tipologia_notizia", resp["items"][0])
        self.assertEqual(resp["items"][0]["tipologia_notizia"], ["notizia"])

    def test_summary_serializer_always_return_servizio_attivo_metadata(self):
        """ """
        servizio = api.content.create(
            container=self.portal,
            type="Servizio",
            title="Test",
        )
        commit()

        resp = self.api_session.get(f"@search?UID={servizio.UID()}").json()
        self.assertEqual(resp["items_total"], 1)
        self.assertIn("servizio_attivo", resp["items"][0])
        self.assertTrue(resp["items"][0]["servizio_attivo"])

        servizio.servizio_attivo = False
        servizio.reindexObject()
        commit()
        resp = self.api_session.get(f"@search?UID={servizio.UID()}").json()
        self.assertFalse(resp["items"][0]["servizio_attivo"])

    def test_summary_serializer_always_return_parent_metadata(self):
        """ """
        parent = api.content.create(
            container=self.portal,
            type="Document",
            title="Parent",
        )
        child = api.content.create(
            container=parent,
            type="Document",
            title="Child",
        )
        commit()

        resp = self.api_session.get(f"@search?UID={child.UID()}").json()
        self.assertIn("parent", resp["items"][0])
        self.assertEqual(resp["items"][0]["parent"]["title"], "Parent")
        self.assertEqual(resp["items"][0]["parent"]["UID"], parent.UID())

    def test_summary_serializer_return_has_children_info_in_GET_calls(self):
        """ """
        news = api.content.create(
            container=self.portal,
            type="News Item",
            title="Test news",
        )

        api.content.create(
            container=news["video"],
            type="Link",
            title="Test link",
        )
        commit()
        resp = self.api_session.get(news.absolute_url()).json()

        self.assertEqual(len(resp["items"]), 3)
        self.assertIn("has_children", resp["items"][0])
        self.assertIn("has_children", resp["items"][1])
        self.assertIn("has_children", resp["items"][2])
        self.assertFalse(resp["items"][0]["has_children"])
        self.assertTrue(resp["items"][1]["has_children"])
        self.assertFalse(resp["items"][2]["has_children"])

    def test_summary_serializer_does_not_return_has_children_info_in_POST_calls(self):
        """ """
        news = api.content.create(
            container=self.portal,
            type="News Item",
            title="Test news",
        )

        api.content.create(
            container=news["video"],
            type="Link",
            title="Test link",
        )
        commit()
        resp = self.api_session.post(
            "@querystring-search",
            json={
                "query": [
                    {
                        "i": "path",
                        "o": "plone.app.querystring.operation.string.absolutePath",
                        "v": f"{news['video'].UID()}::0",
                    }
                ]
            },
        ).json()
        self.assertEqual(len(resp["items"]), 1)
        self.assertNotIn("has_children", resp["items"][0])

        resp = self.api_session.post(
            "@querystring-search",
            json={
                "query": [
                    {
                        "i": "path",
                        "o": "plone.app.querystring.operation.string.absolutePath",
                        "v": f"{news['allegati'].UID()}::0",
                    }
                ]
            },
        ).json()

        self.assertEqual(len(resp["items"]), 1)
        self.assertNotIn("has_children", resp["items"][0])

    def test_pdc_summary_returns_also_contatti_data(self):
        contatti = [{"descrizione": "xxx", "tipo": "email", "valore": "foo@bar.it"}]

        pdc = api.content.create(
            container=self.portal,
            type="PuntoDiContatto",
            title="pdc",
            contatti=contatti,
        )

        serializer = getMultiAdapter((pdc, self.request), ISerializeToJsonSummary)()

        self.assertIn("contatti", serializer)
        self.assertEqual(serializer["contatti"], contatti)

    def test_summary_returns_bando_state_for_bandi_if_required_in_metadata_field(self):
        bando = api.content.create(container=self.portal, type="Bando", title="bando")

        serializer = getMultiAdapter((bando, self.request), ISerializeToJsonSummary)()

        self.assertNotIn("bando_state", serializer)

        self.request.form["metadata_fields"] = "bando_state"
        serializer = getMultiAdapter((bando, self.request), ISerializeToJsonSummary)()

        self.assertIn("bando_state", serializer)



# -*- coding: utf-8 -*-
from collective.z3cform.datagridfield.datagridfield import DataGridFieldFactory
from collective.z3cform.datagridfield.row import DictRow
from iosanita.contenttypes import _
from iosanita.contenttypes.interfaces import IIosanitaContenttypes
from plone.app.z3cform.widget import RelatedItemsFieldWidget
from plone.autoform import directives as form
from plone.supermodel import model
from z3c.relationfield.schema import RelationChoice
from z3c.relationfield.schema import RelationList
from zope import schema


class IPDCValueSchema(model.Schema):
    tipo = schema.Choice(
        title=_("pdc_tipo_label", default="Tipo"),
        # description=_(
        #     "pdc_tipo_help",
        #     default="Tipologia di contatto",
        # ),
        vocabulary="collective.taxonomy.tipologia_pdc",
        required=True,
        default="",
    )
    valore = schema.TextLine(
        title=_("pdc_valore_label", default="Contatto"),
        # description=_(
        #     "pdc_valore_help",
        #     default="Il valore del contatto",
        # ),
        required=True,
        default="",
        max_length=255,
    )
    descrizione = schema.TextLine(
        title=_("pdc_descrizione_label", default="Descrizione"),
        # description=_(
        #     "pdc_descrizione_help",
        #     default="Eventuale descrizione per questo valore",
        # ),
        required=False,
        default="",
        max_length=255,
    )


class IPuntoDiContatto(model.Schema, IIosanitaContenttypes):
    """Marker interface for content type PuntoDiContatto"""

    contatti = schema.List(
        title="Contatti",
        default=[],
        value_type=DictRow(schema=IPDCValueSchema),
        description=_(
            "value_punto_contatto_help",
            default="Il valore del punto di contatto: il numero compreso di prefisso "
            "internazionale (se telefono), l'account (se social network), "
            "l'URL (se sito o pagina web), l'indirizzo email (se email).",
        ),
        required=True,
    )
    uo_correlata = RelationList(
        title=_("uo_correlata_label", default="Unità Organizzativa correlata"),
        description=_(
            "uo_correlata_help",
            default="Selezionare l'Unità Organizzativa per cui questo Punto di contatto è valido. "
            "Se il Punto di contatto è associato ad una Persona, allora quella persona avrà questi contatti nell'Unità organizzativa selezionata.",
        ),
        default=[],
        value_type=RelationChoice(vocabulary="plone.app.vocabularies.Catalog"),
        required=False,
    )

    form.widget(
        "contatti",
        DataGridFieldFactory,
        frontendOptions={
            "widget": "data_grid",
            "widgetProps": {
                "allow_reorder": True,
            },
        },
    )

    form.widget(
        "uo_correlata",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "selectableTypes": ["UnitaOrganizzativa"],
        },
    )



# -*- coding: utf-8 -*-
from plone.app.contenttypes.interfaces import IEvent
from plone.indexer.decorator import indexer


@indexer(IEvent)
def event_location(context, **kw):
    """ """
    strutture_correlate = [x.to_object for x in context.strutture_correlate]
    strutture_correlate = filter(bool, strutture_correlate)
    strutture_correlate_title = [x.UID() for x in strutture_correlate]
    return strutture_correlate_title


@indexer(IEvent)
def rassegna(context, **kw):
    """ """
    children = context.values()
    return "Event" in [child.portal_type for child in children]



# -*- coding: utf-8 -*-
from collective.volto.blocksfield.field import BlocksField
from iosanita.contenttypes import _
from plone.app.dexterity import textindexer
from plone.autoform.interfaces import IFormFieldProvider
from plone.dexterity.interfaces import IDexterityContent
from plone.supermodel import model
from zope.component import adapter
from zope.interface import implementer
from zope.interface import provider


@provider(IFormFieldProvider)
class IUlterioriInformazioni(model.Schema):
    ulteriori_informazioni = BlocksField(
        title=_("ulteriori_informazioni", default="Ulteriori informazioni"),
        description=_(
            "ulteriori_informazioni_help",
            default="Ulteriori informazioni non contemplate dai campi precedenti.",
        ),
        required=False,
    )

    model.fieldset(
        "ulteriori_informazioni",
        label=_("ulteriori_informazioni_label", default="Ulteriori informazioni"),
        fields=["ulteriori_informazioni"],
    )

    textindexer.searchable("ulteriori_informazioni")


@implementer(IUlterioriInformazioni)
@adapter(IDexterityContent)
class UlterioriInformazioni(object):
    """ """

    def __init__(self, context):
        self.context = context



# -*- coding: utf-8 -*-

from . import logger
from plone import api


def upgrade(setup_tool=None):
    """ """
    logger.info(
        "Running upgrade (Python): Change behavior plone.basic => iosanita.basic"
    )

    portal_types = api.portal.get_tool(name="portal_types")

    for ptype in [
        "ComeFarePer",
        "Servizio",
        "Struttura",
        "UnitaOrganizzativa",
    ]:
        behaviors = []
        for behavior in portal_types[ptype].behaviors:
            if behavior == "plone.basic":
                behaviors.append("iosanita.basic")
            else:
                behaviors.append(behavior)
        portal_types[ptype].behaviors = tuple(behaviors)



# -*- coding: utf-8 -*-
from collective.address import _ as _addresmf
from collective.address.behaviors import IAddress
from collective.geolocationbehavior.geolocation import IGeolocatable
from iosanita.contenttypes import _
from plone.app.dexterity import textindexer
from plone.autoform.interfaces import IFormFieldProvider
from plone.dexterity.interfaces import IDexterityContent
from plone.supermodel import model
from zope import schema
from zope.component import adapter
from zope.interface import implementer
from zope.interface import provider


@provider(IFormFieldProvider)
class IDove(IGeolocatable, IAddress):
    nome_sede = schema.TextLine(
        title=_("nome_sede", default="Nome sede"),
        description=_(
            "help_nome_sede",
            default="Inserisci il nome della "
            "sede, se non è presente tra quelle del sito.",
        ),
        required=False,
    )

    provincia = schema.TextLine(
        title=_("provincia", default="Provincia"),
        description=_("help_provincia", default=""),
        required=False,
    )

    circoscrizione = schema.TextLine(
        title=_("circoscrizione", default="Circoscrizione"),
        description=_("help_circoscrizione", default=""),
        required=False,
    )

    # searchabletext indexer
    textindexer.searchable("street")
    textindexer.searchable("nome_sede")
    textindexer.searchable("provincia")
    textindexer.searchable("circoscrizione")
    textindexer.searchable("zip_code")
    textindexer.searchable("city")
    textindexer.searchable("country")

    model.fieldset(
        "dove",
        label=_("dove_label", default="Dove"),
        fields=[
            "nome_sede",
            "street",
            "zip_code",
            "city",
            "provincia",
            "circoscrizione",
            "country",
            "geolocation",
        ],
    )


@provider(IFormFieldProvider)
class IDoveRequired(IDove):
    """redefine some required fields"""

    provincia = schema.TextLine(
        title=_("provincia", default="Provincia"),
        description=_("help_provincia", default=""),
        required=True,
    )
    street = schema.TextLine(
        title=_addresmf("label_street", default="Street"),
        description=_addresmf("help_street", default=""),
        required=True,
    )
    zip_code = schema.TextLine(
        title=_addresmf("label_zip_code", default="Zip Code"),
        description=_addresmf("help_zip_code", default=""),
        required=True,
    )
    city = schema.TextLine(
        title=_addresmf("label_city", default="City"),
        description=_addresmf("help_city", default=""),
        required=True,
    )
    country = schema.Choice(
        title=_addresmf("label_country", default="Country"),
        description=_addresmf(
            "help_country", default="Select the country from the list."
        ),
        required=True,
        vocabulary="collective.address.CountryVocabulary",
    )


@implementer(IDove)
@adapter(IDexterityContent)
class Dove(object):
    """ """

    def __init__(self, context):
        self.context = context


@implementer(IDoveRequired)
@adapter(IDexterityContent)
class DoveRequired(object):
    """ """

    def __init__(self, context):
        self.context = context



# -*- coding: utf-8 -*-
from iosanita.contenttypes.interfaces.persona import IPersona
from plone.app.content.interfaces import INameFromTitle
from plone.dexterity.content import Container
from zope.interface import implementer


@implementer(IPersona, INameFromTitle)
class Persona(Container):
    """Persona CT"""

    @property
    def title(self):
        """
        Title is set from nome and cognome fields
        """
        nome = getattr(self, "nome", "")
        cognome = getattr(self, "cognome", "")
        titolo_persona = getattr(self, "titolo_persona", "")
        return " ".join([p for p in [titolo_persona, cognome, nome] if p])

    @title.setter
    def title(self, value):
        pass

    # maybe needed when we enable rubrica
    # @property
    # def rubrica_title(self):
    #     if getattr(self, "nome", "") and getattr(self, "cognome", ""):
    #         return "{cognome} {nome}".format(nome=self.nome, cognome=self.cognome)
    #     else:
    #         return ""

    # @property
    # def rubrica_id(self):
    #     return idnormalizer.normalize(self.rubrica_title)



# -*- coding: utf-8 -*-
from collective.volto.blocksfield.field import BlocksField
from iosanita.contenttypes import _
from plone.app.dexterity import textindexer
from plone.app.event.base import default_timezone
from plone.app.z3cform.widget import DatetimeFieldWidget
from plone.autoform import directives as form
from plone.autoform.interfaces import IFormFieldProvider
from plone.dexterity.interfaces import IDexterityContent
from plone.supermodel import model
from zope import schema
from zope.component import adapter
from zope.interface import implementer
from zope.interface import provider


@provider(IFormFieldProvider)
class IBando(model.Schema):
    """Marker interface for content type Bando"""

    descrizione_estesa = BlocksField(
        title=_("descrizione_estesa_label", default="Cos'è"),
        required=True,
        description=_(
            "descrizione_estesa_bando_help",
            default="Descrizione dettagliata e completa.",
        ),
    )

    come_partecipare = BlocksField(
        title=_("come_partecipare_label", default="Come partecipare"),
        required=True,
        description=_(
            "come_partecipare_help",
            default="Descrizione delle modalità di partecipazione al bando o al concorso e collegamento.",
        ),
    )
    modalita_selezione = BlocksField(
        title=_("modalita_selezione_label", default="Modalità di selezione"),
        required=True,
        description=_(
            "modalita_selezione_help",
            default="Informazioni sulle modalità di selezione, ad esempio da quante prove e di che tipo è composta la selezione.",
        ),
    )

    scadenza_domande_bando = schema.Datetime(
        title=_(
            "scadenza_domande_bando_label",
            default="Termine per le richieste di chiarimenti",
        ),
        description=_(
            "scadenza_domande_bando_help",
            default="Data entro la quale sarà possibile far pervenire domande"
            " e richieste di chiarimento a chi eroga il bando",
        ),
        required=False,
    )

    note_aggiornamento = schema.TextLine(
        title=_("note_aggiornamento_label", default="Note di aggiornamento"),
        description=_(
            "help_note_aggiornamento",
            default="Inserisci una nota per indicare che il contenuto corrente è stato aggiornato."  # noqa
            " Questo testo può essere visualizzato nei blocchi elenco con determinati layout per informare "  # noqa
            "gli utenti che un determinato contenuto è stato aggiornato. "
            "Ad esempio se in un bando sono stati aggiunti dei documenti.",
        ),
        required=False,
    )

    # custom fieldsets and order
    model.fieldset(
        "cosa_e",
        label=_("cosa_e_fieldset", default="Cos'è"),
        fields=[
            "descrizione_estesa",
        ],
    )
    model.fieldset(
        "come_partecipare",
        label=_("come_partecipare_label", default="Come partecipare"),
        fields=[
            "come_partecipare",
        ],
    )
    model.fieldset(
        "modalita_selezione",
        label=_("modalita_selezione_label", default="Modalità di selezione"),
        fields=[
            "modalita_selezione",
        ],
    )

    # custom widgets
    form.widget(
        "scadenza_domande_bando",
        DatetimeFieldWidget,
        default_timezone=default_timezone,
    )

    # custom order
    form.order_after(scadenza_domande_bando="apertura_bando")

    textindexer.searchable("descrizione_estesa")


@implementer(IBando)
@adapter(IDexterityContent)
class Bando(object):
    """ """

    def __init__(self, context):
        self.context = context



# -*- coding: utf-8 -*-
"""Setup tests for this package."""
from iosanita.contenttypes.testing import INTEGRATION_TESTING
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID

import unittest


class TestTaxonomiesCustomMetadata(unittest.TestCase):
    """"""

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]

        self.request["LANGUAGE"] = "it"

        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

    def test_parliamo_di_metadata_has_array_of_objects(self):
        """ """
        struttura = api.content.create(
            container=self.portal,
            type="Struttura",
            title="Test servizio",
            parliamo_di=["ticket-ed-esenzioni", "igiene-pubblica"],
        )
        res = api.content.find(UID=struttura.UID())

        self.assertEqual(len(res), 1)
        self.assertEqual(
            res[0].parliamo_di, sorted(["ticket-ed-esenzioni", "igiene-pubblica"])
        )
        self.assertEqual(
            res[0].parliamo_di_metadata,
            [
                {"title": "Ticket ed esenzioni", "token": "ticket-ed-esenzioni"},
                {"title": "Igiene Pubblica", "token": "igiene-pubblica"},
            ],
        )

    def test_a_chi_si_rivolge_tassonomia_metadata_has_array_of_objects(self):
        """ """
        struttura = api.content.create(
            container=self.portal,
            type="Struttura",
            title="Test servizio",
            a_chi_si_rivolge_tassonomia=[
                "farmacie",
                "imprese",
            ],
        )
        res = api.content.find(UID=struttura.UID())

        self.assertEqual(len(res), 1)
        self.assertEqual(
            res[0].a_chi_si_rivolge_tassonomia,
            sorted(["farmacie", "imprese"]),
        )
        self.assertEqual(
            res[0].a_chi_si_rivolge_tassonomia_metadata,
            [
                {"title": "Farmacie", "token": "farmacie"},
                {"title": "Imprese", "token": "imprese"},
            ],
        )

    def test_incarico_has_last_taxonomy_leaf_value(self):
        """ """
        persona = api.content.create(
            container=self.portal,
            type="Persona",
            title="john doe",
            incarico=[
                "medico",
            ],
        )
        res = api.content.find(UID=persona.UID())[0]

        self.assertEqual(
            res.incarico_metadata,
            [{"title": "Medico", "token": "medico"}],
        )

    def test_tipologia_notizia_has_last_taxonomy_leaf_value(self):
        """ """
        news = api.content.create(
            container=self.portal,
            type="News Item",
            title="xxx",
            tipologia_notizia=[
                "comunicato-stampa",
            ],
        )
        res = api.content.find(UID=news.UID())[0]

        self.assertEqual(
            res.tipologia_notizia_metadata,
            [{"title": "Comunicato (stampa)", "token": "comunicato-stampa"}],
        )

    def test_tipologia_servizio_has_last_taxonomy_leaf_value(self):
        """ """
        servizio = api.content.create(
            container=self.portal,
            type="Servizio",
            title="xxx",
            tipologia_servizio=[
                "prevenzione-e-vaccini",
            ],
        )
        res = api.content.find(UID=servizio.UID())[0]

        self.assertEqual(
            res.tipologia_servizio_metadata,
            [{"title": "Prevenzione e vaccini", "token": "prevenzione-e-vaccini"}],
        )



from plone.autoform.interfaces import IFormFieldProvider
from plone.autoform.interfaces import OMITTED_KEY
from plone.supermodel.interfaces import ISchemaPlugin
from redturtle.bandi.interfaces.bandoSchema import IBandoSchema
from zope.component import adapter
from zope.interface import implementer
from zope.interface import Interface

import os


@implementer(ISchemaPlugin)
@adapter(IFormFieldProvider)
class SchemaTweaks(object):
    """ """

    order = 999999

    def __init__(self, schema):
        self.schema = schema

    def __call__(self):
        self.fix_bando()

    def fix_bando(self):
        """fix Bando fields"""
        if os.environ.get("DISABLE_VALIDATION_DURING_IMPORT") in ["1", "true", "True"]:
            return
        if self.schema.getName() == "IBando":
            # set it only once
            IBandoSchema["destinatari"].required = False
            IBandoSchema["scadenza_bando"].required = True
            IBandoSchema.setTaggedValue(
                OMITTED_KEY,
                [(Interface, "destinatari", "true")],
            )



from iosanita.contenttypes import _
from iosanita.contenttypes.interfaces.settings import IIoSanitaContenttypesSettings
from plone.app.registry.browser import controlpanel


class IoSanitaContenttypesSettingsForm(controlpanel.RegistryEditForm):
    schema = IIoSanitaContenttypesSettings
    label = _(
        "iosanita_contenttypes_settings_label",
        default="Impostazioni IoSanita Contenty-types",
    )


class IoSanitaContenttypesControlPanel(controlpanel.ControlPanelFormWrapper):
    form = IoSanitaContenttypesSettingsForm



# -*- coding: utf-8 -*-
"""Setup tests for this package."""
from iosanita.contenttypes.testing import INTEGRATION_TESTING
from iosanita.contenttypes.testing import RESTAPI_TESTING
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.restapi.interfaces import ISerializeToJson
from plone.restapi.interfaces import ISerializeToJsonSummary
from plone.restapi.testing import RelativeSession
from Products.CMFPlone.interfaces import ISelectableConstrainTypes
from zope.component import queryMultiAdapter

import unittest


class TestNewsSchema(unittest.TestCase):
    layer = RESTAPI_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)

    def tearDown(self):
        self.api_session.close()

    def test_behaviors_enabled_for_bando(self):
        portal_types = api.portal.get_tool(name="portal_types")
        self.assertEqual(
            portal_types["News Item"].behaviors,
            (
                "plone.dublincore",
                "plone.namefromtitle",
                "plone.allowdiscussion",
                "plone.shortname",
                "plone.excludefromnavigation",
                "plone.relateditems",
                "plone.leadimage",
                "plone.versioning",
                "plone.locking",
                "volto.preview_image",
                "plone.constraintypes",
                "plone.textindexer",
                "plone.translatable",
                "kitconcept.seo",
                "collective.taxonomy.generated.tipologia_notizia",
                "collective.taxonomy.generated.parliamo_di",
                "iosanita.contenttypes.behavior.news",
            ),
        )

    def test_news_fieldsets(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/News Item").json()
        self.assertEqual(len(resp["fieldsets"]), 7)
        self.assertEqual(
            [x.get("id") for x in resp["fieldsets"]],
            [
                "default",
                "correlati",
                "dates",
                "categorization",
                "ownership",
                "settings",
                "seo",
            ],
        )

    def test_news_required_fields(self):
        resp = self.api_session.get("@types/News Item").json()
        self.assertEqual(
            sorted(resp["required"]),
            sorted(
                [
                    "descrizione_estesa",
                    "title",
                    "uo_correlata",
                ]
            ),
        )

    def test_news_fields_default_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/News Item").json()
        self.assertEqual(
            resp["fieldsets"][0]["fields"],
            [
                "title",
                "description",
                "numero_progressivo_cs",
                "descrizione_estesa",
                "image",
                "image_caption",
                "preview_image",
                "preview_caption",
                "tipologia_notizia",
                "parliamo_di",
            ],
        )

    def test_news_fields_correlati_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/News Item").json()
        self.assertEqual(
            resp["fieldsets"][1]["fields"],
            [
                "persona_correlata",
                "struttura_correlata",
                "servizio_correlato",
                "uo_correlata",
                "notizia_correlata",
            ],
        )


class TestNews(unittest.TestCase):
    """"""

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.request["LANGUAGE"] = "it"

        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

    def test_news_default_children(self):
        news = api.content.create(container=self.portal, type="News Item", title="xxx")

        self.assertEqual(news.keys(), ["immagini", "video", "allegati"])

        self.assertEqual("Document", news["immagini"].portal_type)
        self.assertEqual("Document", news["video"].portal_type)
        self.assertEqual("Document", news["allegati"].portal_type)

    def test_news_immagini_has_filtered_addable_types(self):
        news = api.content.create(container=self.portal, type="News Item", title="xxx")
        immagini = ISelectableConstrainTypes(news["immagini"])
        self.assertEqual(immagini.getConstrainTypesMode(), 1)
        self.assertEqual(immagini.getLocallyAllowedTypes(), ["Link", "Image"])

    def test_news_video_has_filtered_addable_types(self):
        news = api.content.create(container=self.portal, type="News Item", title="xxx")
        video = ISelectableConstrainTypes(news["video"])
        self.assertEqual(video.getConstrainTypesMode(), 1)
        self.assertEqual(video.getLocallyAllowedTypes(), ["Link"])

    def test_news_allegati_has_filtered_addable_types(self):
        news = api.content.create(container=self.portal, type="News Item", title="xxx")
        allegati = ISelectableConstrainTypes(news["allegati"])
        self.assertEqual(allegati.getConstrainTypesMode(), 1)
        self.assertEqual(allegati.getLocallyAllowedTypes(), ["File"])

    def test_news_type_title_based_on_tipologia_notizia(self):
        news = api.content.create(
            container=self.portal,
            type="News Item",
            title="xxx",
            tipologia_notizia="notizia",
        )

        serializer = queryMultiAdapter((news, self.request), ISerializeToJson)()

        self.assertEqual(serializer["type_title"], "Notizia")

        news.tipologia_notizia = "comunicato-stampa"

        serializer = queryMultiAdapter((news, self.request), ISerializeToJson)()

        self.assertEqual(serializer["type_title"], "Comunicato (stampa)")

    def test_news_type_title_based_on_tipologia_notizia_on_brains(self):
        news = api.content.create(
            container=self.portal,
            type="News Item",
            title="xxx",
            tipologia_notizia="notizia",
        )

        brain = api.content.find(UID=news.UID())[0]
        serializer = queryMultiAdapter((brain, self.request), ISerializeToJsonSummary)()

        self.assertEqual(serializer["type_title"], "Notizia")

        news.tipologia_notizia = "comunicato-stampa"
        news.reindexObject()

        brain = api.content.find(UID=news.UID())[0]
        serializer = queryMultiAdapter((brain, self.request), ISerializeToJsonSummary)()

        self.assertEqual(serializer["type_title"], "Comunicato (stampa)")

    def test_news_type_title_default_if_wrong_tipologia_notizia(self):
        news = api.content.create(
            container=self.portal,
            type="News Item",
            title="xxx",
            tipologia_notizia="xxx",
        )

        serializer = queryMultiAdapter((news, self.request), ISerializeToJson)()

        self.assertEqual(serializer["type_title"], "Notizia")

    def test_news_type_title_default_if_wrong_tipologia_notizia_on_brains(self):
        news = api.content.create(
            container=self.portal,
            type="News Item",
            title="xxx",
            tipologia_notizia="xxx",
        )

        brain = api.content.find(UID=news.UID())[0]
        serializer = queryMultiAdapter((brain, self.request), ISerializeToJsonSummary)()

        self.assertEqual(serializer["type_title"], "Notizia")



# -*- coding: utf-8 -*-
from iosanita.contenttypes import _
from plone.app.dexterity import textindexer
from plone.autoform import directives as form
from plone.autoform.interfaces import IFormFieldProvider
from plone.dexterity.interfaces import IDexterityContent
from plone.supermodel import model
from zope import schema
from zope.component import adapter
from zope.interface import implementer
from zope.interface import provider


@provider(IFormFieldProvider)
class ISottotitolo(model.Schema):
    """Marker interface"""

    sottotitolo = schema.TextLine(
        title=_("sottotitolo_label", default="Sottotitolo"),
        description=_(
            "sottotitolo_help",
            default="Indica un eventuale sottotitolo/titolo alternativo.",
        ),
        required=False,
    )
    form.order_after(sottotitolo="IBasic.description")

    # SearchableText
    textindexer.searchable("sottotitolo")


@implementer(ISottotitolo)
@adapter(IDexterityContent)
class Sottotitolo(object):
    """"""

    def __init__(self, context):
        self.context = context
